#!/usr/bin/python
# -*- coding: utf-8 -*-

#
# Dell EMC OpenManage Ansible Modules
# Version 1.2
# Copyright (C) 2019 Dell Inc.

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# All rights reserved. Dell, EMC, and other trademarks are trademarks of Dell Inc. or its subsidiaries.
# Other trademarks may be trademarks of their respective owners.
#


from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
---
module: ome_device_info
short_description: Retrieves the information about Device.
description:
   - This module retrieves the list of all devices information with the exhaustive inventory of each
     device.
options:
    hostname:
        description:
            - Target IP Address or hostname.
        type: str
        required: True
    username:
        description:
            - Target username.
        type: str
        required: True
    password:
        description:
            - Target user password.
        type: str
        required: True
    port:
        description:
            - Target HTTPS port.
        type: int
        default: 443
    fact_subset:
        description:
            - C(basic_inventory) returns the list of the devices.
            - C(detailed_inventory) returns the inventory details of specified devices.
            - C(subsystem_health) returns the health status of specified devices.
        type: str
        choices: [basic_inventory, detailed_inventory, subsystem_health ]
        default: basic_inventory
    system_query_options:
        description:
            - I(system_query_options) applicable for the choices of the fact_subset. Either I(device_id) or I(device_service_tag)
              is mandatory for C(detailed_inventory) and C(subsystem_health) or both can be applicable.
        type: dict
        suboptions:
            device_id:
                 description:
                    - A list of unique identifier is applicable
                      for C(detailed_inventory) and C(subsystem_health).
                 type: list
            device_service_tag:
                 description:
                    - A list of service tags are applicable for C(detailed_inventory)
                      and C(subsystem_health).
                 type: list
            inventory_type:
                description:
                    - For C(detailed_inventory), it returns details of the specified inventory type.
                type: str
            filter:
                description:
                    - For C(basic_inventory), it filters the collection of devices.
                      I(filter) query format should be aligned with OData standards.
                type: str

requirements:
    - "python >= 2.7.5"
author: "Sajna Shetty(@Sajna-Shetty)"
'''

EXAMPLES = """
---
- name: Retrieve basic inventory of all devices.
  community.general.ome_device_info:
    hostname: "192.168.0.1"
    username: "username"
    password: "password"

- name: Retrieve basic inventory for devices identified by IDs 33333 or 11111 using filtering.
  community.general.ome_device_info:
    hostname: "192.168.0.1"
    username: "username"
    password: "password"
    fact_subset: "basic_inventory"
    system_query_options:
      filter: "Id eq 33333 or Id eq 11111"

- name: Retrieve inventory details of specified devices identified by IDs 11111 and 22222.
  community.general.ome_device_info:
    hostname: "192.168.0.1"
    username: "username"
    password: "password"
    fact_subset: "detailed_inventory"
    system_query_options:
      device_id:
        - 11111
        - 22222

- name: Retrieve inventory details of specified devices identified by service tags MXL1234 and MXL4567.
  community.general.ome_device_info:
    hostname: "192.168.0.1"
    username: "username"
    password: "password"
    fact_subset: "detailed_inventory"
    system_query_options:
      device_service_tag:
        - MXL1234
        - MXL4567

- name: Retrieve details of specified inventory type of specified devices identified by ID and service tags.
  community.general.ome_device_info:
    hostname: "192.168.0.1"
    username: "username"
    password: "password"
    fact_subset: "detailed_inventory"
    system_query_options:
      device_id:
        - 11111
      device_service_tag:
        - MXL1234
        - MXL4567
      inventory_type: "serverDeviceCards"

- name: Retrieve subsystem health of specified devices identified by service tags.
  community.general.ome_device_info:
    hostname: "192.168.0.1"
    username: "username"
    password: "password"
    fact_subset: "subsystem_health"
    system_query_options:
      device_service_tag:
        - MXL1234
        - MXL4567

"""

RETURN = '''
---
msg:
  type: str
  description: Over all device information status.
  returned: on error
  sample: "Failed to fetch the device information"
device_info:
  type: dict
  description: Returns the information collected from the Device.
  returned: success
  sample: {
  "value": [
        {
            "Actions": null,
            "AssetTag": null,
            "ChassisServiceTag": null,
            "ConnectionState": true,
            "DeviceManagement": [
                {
                    "DnsName": "dnsname.host.com",
                    "InstrumentationName": "MX-12345",
                    "MacAddress": "11:10:11:10:11:10",
                    "ManagementId": 12345,
                    "ManagementProfile": [
                        {
                            "HasCreds": 0,
                            "ManagementId": 12345,
                            "ManagementProfileId": 12345,
                            "ManagementURL": "https://192.168.0.1:443",
                            "Status": 1000,
                            "StatusDateTime": "2019-01-21 06:30:08.501"
                        }
                    ],
                    "ManagementType": 2,
                    "NetworkAddress": "192.168.0.1"
                }
            ],
            "DeviceName": "MX-0003I",
            "DeviceServiceTag": "MXL1234",
            "DeviceSubscription": null,
            "LastInventoryTime": "2019-01-21 06:30:08.501",
            "LastStatusTime": "2019-01-21 06:30:02.492",
            "ManagedState": 3000,
            "Model": "PowerEdge MX7000",
            "PowerState": 17,
            "SlotConfiguration": {},
            "Status": 4000,
            "SystemId": 2031,
            "Type": 2000
        }
    ]
  }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.remote_management.dellemc.ome import RestOME
from ansible.module_utils.six.moves.urllib.error import URLError, HTTPError
from ansible.module_utils.urls import ConnectionError, SSLValidationError

DEVICES_INVENTORY_DETAILS = "detailed_inventory"
DEVICES_SUBSYSTEM_HEALTH = "subsystem_health"
DEVICES_INVENTORY_TYPE = "inventory_type"
DEVICE_LIST = "basic_inventory"
DESC_HTTP_ERROR = "HTTP Error 404: Not Found"
device_fact_error_report = {}

DEVICE_RESOURCE_COLLECTION = {
    DEVICE_LIST: {"resource": "DeviceService/Devices"},
    DEVICES_INVENTORY_DETAILS: {"resource": "DeviceService/Devices({Id})/InventoryDetails"},
    DEVICES_INVENTORY_TYPE: {"resource": "DeviceService/Devices({Id})/InventoryDetails('{InventoryType}')"},
    DEVICES_SUBSYSTEM_HEALTH: {"resource": "DeviceService/Devices({Id})/SubSystemHealth"},
}


def _get_device_id_from_service_tags(service_tags, rest_obj):
    """
    Get device ids from device service tag
    Returns :dict : device_id to service_tag map
    :arg service_tags: service tag
    :arg rest_obj: RestOME class object in case of request with session.
    :returns: dict eg: {1345:"MXL1245"}
    """
    try:
        path = DEVICE_RESOURCE_COLLECTION[DEVICE_LIST]["resource"]
        resp = rest_obj.invoke_request('GET', path)
        if resp.success:
            devices_list = resp.json_data["value"]
            service_tag_dict = {}
            for item in devices_list:
                if item["DeviceServiceTag"] in service_tags:
                    service_tag_dict.update({item["Id"]: item["DeviceServiceTag"]})
            available_service_tags = service_tag_dict.values()
            not_available_service_tag = list(set(service_tags) - set(available_service_tags))
            device_fact_error_report.update(dict((tag, DESC_HTTP_ERROR) for tag in not_available_service_tag))
        else:
            raise ValueError(resp.json_data)
    except (URLError, HTTPError, SSLValidationError, ConnectionError, TypeError, ValueError) as err:
        raise err
    return service_tag_dict


def is_int(val):
    """check when device_id numeric represented value is int"""
    try:
        int(val)
        return True
    except ValueError:
        return False


def _check_duplicate_device_id(device_id_list, service_tag_dict):
    """If service_tag is duplicate of device_id, then updates the message as Duplicate report
    :arg1: device_id_list : list of device_id
    :arg2: service_tag_id_dict: dictionary of device_id to service tag map"""
    if device_id_list:
        device_id_represents_int = [int(device_id) for device_id in device_id_list if device_id and is_int(device_id)]
        common_val = list(set(device_id_represents_int) & set(service_tag_dict.keys()))
        for device_id in common_val:
            device_fact_error_report.update(
                {service_tag_dict[device_id]: "Duplicate report of device_id: {0}".format(device_id)})
            del service_tag_dict[device_id]


def _get_device_identifier_map(module_params, rest_obj):
    """
    Builds the identifiers mapping
    :returns: the dict of device_id to server_tag map
     eg: {"device_id":{1234: None},"device_service_tag":{1345:"MXL1234"}}"""
    system_query_options_param = module_params.get("system_query_options")
    device_id_service_tag_dict = {}
    if system_query_options_param is not None:
        device_id_list = system_query_options_param.get("device_id")
        device_service_tag_list = system_query_options_param.get("device_service_tag")
        if device_id_list:
            device_id_dict = dict((device_id, None) for device_id in list(set(device_id_list)))
            device_id_service_tag_dict["device_id"] = device_id_dict
        if device_service_tag_list:
            service_tag_dict = _get_device_id_from_service_tags(device_service_tag_list,
                                                                rest_obj)

            _check_duplicate_device_id(device_id_list, service_tag_dict)
            device_id_service_tag_dict["device_service_tag"] = service_tag_dict
    return device_id_service_tag_dict


def _get_query_parameters(module_params):
    """
    Builds query parameter
    :returns: dictionary, which is applicable builds the query format
     eg : {"$filter":"Type eq 2000"}
     """
    system_query_options_param = module_params.get("system_query_options")
    query_parameter = None
    if system_query_options_param:
        filter_by_val = system_query_options_param.get("filter")
        if filter_by_val:
            query_parameter = {"$filter": filter_by_val}
    return query_parameter


def _get_resource_parameters(module_params, rest_obj):
    """
    Identifies the resource path by different states
    :returns: dictionary containing identifier with respective resource path
        eg:{"device_id":{1234:""DeviceService/Devices(1234)/InventoryDetails"},
        "device_service_tag":{"MXL1234":"DeviceService/Devices(1345)/InventoryDetails"}}
    """
    fact_subset = module_params["fact_subset"]
    path_dict = {}
    if fact_subset != DEVICE_LIST:
        inventory_type = None
        device_id_service_tag_dict = _get_device_identifier_map(module_params, rest_obj)
        if fact_subset == DEVICES_INVENTORY_DETAILS:
            system_query_options = module_params.get("system_query_options")
            inventory_type = system_query_options.get(DEVICES_INVENTORY_TYPE)
        path_identifier = DEVICES_INVENTORY_TYPE if inventory_type else fact_subset
        for identifier_type, identifier_dict in device_id_service_tag_dict.items():
            path_dict[identifier_type] = {}
            for device_id, service_tag in identifier_dict.items():
                key_identifier = service_tag if identifier_type == "device_service_tag" else device_id
                path = DEVICE_RESOURCE_COLLECTION[path_identifier]["resource"].format(Id=device_id,
                                                                                      InventoryType=inventory_type)
                path_dict[identifier_type].update({key_identifier: path})
    else:
        path_dict.update({DEVICE_LIST: DEVICE_RESOURCE_COLLECTION[DEVICE_LIST]["resource"]})
    return path_dict


def _check_mutually_inclusive_arguments(val, module_params, required_args):
    """"
     Throws error if arguments detailed_inventory, subsystem_health
     not exists with qualifier device_id or device_service_tag"""
    system_query_options_param = module_params.get("system_query_options")
    if system_query_options_param is None or (system_query_options_param is not None and not any(
            system_query_options_param.get(qualifier) for qualifier in required_args)):
        raise ValueError("One of the following {0} is required for {1}".format(required_args, val))


def _validate_inputs(module_params):
    """validates input parameters"""
    fact_subset = module_params["fact_subset"]
    if fact_subset != "basic_inventory":
        _check_mutually_inclusive_arguments(fact_subset, module_params, ["device_id", "device_service_tag"])


def main():
    system_query_options = {"type": 'dict', "required": False, "options": {
        "device_id": {"type": 'list'},
        "device_service_tag": {"type": 'list'},
        "inventory_type": {"type": 'str'},
        "filter": {"type": 'str', "required": False},
    }}

    module = AnsibleModule(
        argument_spec={
            "hostname": {"required": True, "type": 'str'},
            "username": {"required": True, "type": 'str'},
            "password": {"required": True, "type": 'str', "no_log": True},
            "port": {"required": False, "default": 443, "type": 'int'},
            "fact_subset": {"required": False, "default": "basic_inventory",
                            "choices": ['basic_inventory', 'detailed_inventory', 'subsystem_health']},
            "system_query_options": system_query_options,
        },
        required_if=[['fact_subset', 'detailed_inventory', ['system_query_options']],
                     ['fact_subset', 'subsystem_health', ['system_query_options']], ],
        supports_check_mode=False)

    try:
        _validate_inputs(module.params)
        with RestOME(module.params, req_session=True) as rest_obj:
            device_facts = _get_resource_parameters(module.params, rest_obj)
            resp_status = []
            if device_facts.get("basic_inventory"):
                query_param = _get_query_parameters(module.params)
                resp = rest_obj.invoke_request('GET', device_facts["basic_inventory"], query_param=query_param)
                device_facts = resp.json_data
                resp_status.append(resp.status_code)
            else:
                for identifier_type, path_dict_map in device_facts.items():
                    for identifier, path in path_dict_map.items():
                        try:
                            resp = rest_obj.invoke_request('GET', path)
                            data = resp.json_data
                            resp_status.append(resp.status_code)
                        except HTTPError as err:
                            data = str(err)
                        path_dict_map[identifier] = data
                if any(device_fact_error_report):
                    if "device_service_tag" in device_facts:
                        device_facts["device_service_tag"].update(device_fact_error_report)
                    else:
                        device_facts["device_service_tag"] = device_fact_error_report
        if 200 in resp_status:
            module.exit_json(device_info=device_facts)
        else:
            module.fail_json(msg="Failed to fetch the device information")
    except (URLError, HTTPError, SSLValidationError, ConnectionError, TypeError, ValueError) as err:
        module.fail_json(msg=str(err))


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# (c) 2018, Jean-Philippe Evrard <jean-philippe@evrard.me>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: etcd3
short_description: "Set or delete key value pairs from an etcd3 cluster"
requirements:
  - etcd3
description:
   - Sets or deletes values in etcd3 cluster using its v3 api.
   - Needs python etcd3 lib to work
options:
    key:
        description:
            - the key where the information is stored in the cluster
        required: true
    value:
        description:
            - the information stored
        required: true
    host:
        description:
            - the IP address of the cluster
        default: 'localhost'
    port:
        description:
            - the port number used to connect to the cluster
        default: 2379
    state:
        description:
            - the state of the value for the key.
            - can be present or absent
        required: true
    user:
        description:
            - The etcd user to authenticate with.
    password:
        description:
            - The password to use for authentication.
            - Required if I(user) is defined.
    ca_cert:
        description:
            - The Certificate Authority to use to verify the etcd host.
            - Required if I(client_cert) and I(client_key) are defined.
    client_cert:
        description:
            - PEM formatted certificate chain file to be used for SSL client authentication.
            - Required if I(client_key) is defined.
    client_key:
        description:
            - PEM formatted file that contains your private key to be used for SSL client authentication.
            - Required if I(client_cert) is defined.
    timeout:
        description:
            - The socket level timeout in seconds.
author:
    - Jean-Philippe Evrard (@evrardjp)
    - Victor Fauth (@vfauth)
'''

EXAMPLES = """
- name: Store a value "bar" under the key "foo" for a cluster located "http://localhost:2379"
  community.general.etcd3:
    key: "foo"
    value: "baz3"
    host: "localhost"
    port: 2379
    state: "present"

- name: Authenticate using user/password combination with a timeout of 10 seconds
  community.general.etcd3:
    key: "foo"
    value: "baz3"
    state: "present"
    user: "someone"
    password: "password123"
    timeout: 10

- name: Authenticate using TLS certificates
  community.general.etcd3:
    key: "foo"
    value: "baz3"
    state: "present"
    ca_cert: "/etc/ssl/certs/CA_CERT.pem"
    client_cert: "/etc/ssl/certs/cert.crt"
    client_key: "/etc/ssl/private/key.pem"
"""

RETURN = '''
key:
    description: The key that was queried
    returned: always
    type: str
old_value:
    description: The previous value in the cluster
    returned: always
    type: str
'''

import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils._text import to_native


try:
    import etcd3
    HAS_ETCD = True
except ImportError:
    ETCD_IMP_ERR = traceback.format_exc()
    HAS_ETCD = False


def run_module():
    # define the available arguments/parameters that a user can pass to
    # the module
    module_args = dict(
        key=dict(type='str', required=True),
        value=dict(type='str', required=True),
        host=dict(type='str', default='localhost'),
        port=dict(type='int', default=2379),
        state=dict(type='str', required=True, choices=['present', 'absent']),
        user=dict(type='str'),
        password=dict(type='str', no_log=True),
        ca_cert=dict(type='path'),
        client_cert=dict(type='path'),
        client_key=dict(type='path'),
        timeout=dict(type='int'),
    )

    # seed the result dict in the object
    # we primarily care about changed and state
    # change is if this module effectively modified the target
    # state will include any data that you want your module to pass back
    # for consumption, for example, in a subsequent task
    result = dict(
        changed=False,
    )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True,
        required_together=[['client_cert', 'client_key'], ['user', 'password']],
    )

    # It is possible to set `ca_cert` to verify the server identity without
    # setting `client_cert` or `client_key` to authenticate the client
    # so required_together is enough
    # Due to `required_together=[['client_cert', 'client_key']]`, checking the presence
    # of either `client_cert` or `client_key` is enough
    if module.params['ca_cert'] is None and module.params['client_cert'] is not None:
        module.fail_json(msg="The 'ca_cert' parameter must be defined when 'client_cert' and 'client_key' are present.")

    result['key'] = module.params.get('key')
    module.params['cert_cert'] = module.params.pop('client_cert')
    module.params['cert_key'] = module.params.pop('client_key')

    if not HAS_ETCD:
        module.fail_json(msg=missing_required_lib('etcd3'), exception=ETCD_IMP_ERR)

    allowed_keys = ['host', 'port', 'ca_cert', 'cert_cert', 'cert_key',
                    'timeout', 'user', 'password']
    # TODO(evrardjp): Move this back to a dict comprehension when python 2.7 is
    # the minimum supported version
    # client_params = {key: value for key, value in module.params.items() if key in allowed_keys}
    client_params = dict()
    for key, value in module.params.items():
        if key in allowed_keys:
            client_params[key] = value
    try:
        etcd = etcd3.client(**client_params)
    except Exception as exp:
        module.fail_json(msg='Cannot connect to etcd cluster: %s' % (to_native(exp)),
                         exception=traceback.format_exc())
    try:
        cluster_value = etcd.get(module.params['key'])
    except Exception as exp:
        module.fail_json(msg='Cannot reach data: %s' % (to_native(exp)),
                         exception=traceback.format_exc())

    # Make the cluster_value[0] a string for string comparisons
    result['old_value'] = to_native(cluster_value[0])

    if module.params['state'] == 'absent':
        if cluster_value[0] is not None:
            if module.check_mode:
                result['changed'] = True
            else:
                try:
                    etcd.delete(module.params['key'])
                except Exception as exp:
                    module.fail_json(msg='Cannot delete %s: %s' % (module.params['key'], to_native(exp)),
                                     exception=traceback.format_exc())
                else:
                    result['changed'] = True
    elif module.params['state'] == 'present':
        if result['old_value'] != module.params['value']:
            if module.check_mode:
                result['changed'] = True
            else:
                try:
                    etcd.put(module.params['key'], module.params['value'])
                except Exception as exp:
                    module.fail_json(msg='Cannot add or edit key %s: %s' % (module.params['key'], to_native(exp)),
                                     exception=traceback.format_exc())
                else:
                    result['changed'] = True
    else:
        module.fail_json(msg="State not recognized")

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)

    # during the execution of the module, if there is an exception or a
    # conditional state that effectively causes a failure, run
    # AnsibleModule.fail_json() to pass in the message and the result

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    module.exit_json(**result)


def main():
    run_module()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ce_vrf_af
short_description: Manages VPN instance address family on HUAWEI CloudEngine switches.
description:
    - Manages VPN instance address family of HUAWEI CloudEngine switches.
author: Yang yang (@QijunPan)
notes:
    - If I(state=absent), the vrf will be removed, regardless of the non-required parameters.
    - This module requires the netconf system service be enabled on the remote device being managed.
    - Recommended connection is C(netconf).
    - This module also works with C(local) connections for legacy playbooks.
options:
    vrf:
        description:
            - VPN instance.
        required: true
    vrf_aftype:
        description:
            - VPN instance address family.
        choices: ['v4','v6']
        default: v4
    route_distinguisher:
        description:
            - VPN instance route distinguisher,the RD used to distinguish same route prefix from different vpn.
              The RD must be setted before setting vpn_target_value.
    vpn_target_state:
        description:
            - Manage the state of the vpn target.
        choices: ['present','absent']
    vpn_target_type:
        description:
            - VPN instance vpn target type.
        choices: ['export_extcommunity', 'import_extcommunity']
    vpn_target_value:
        description:
            - VPN instance target value. Such as X.X.X.X:number<0-65535> or number<0-65535>:number<0-4294967295>
              or number<0-65535>.number<0-65535>:number<0-65535> or number<65536-4294967295>:number<0-65535>
              but not support 0:0 and 0.0:0.
    evpn:
        description:
            - Is extend vpn or normal vpn.
        type: bool
        default: 'no'
    state:
        description:
            - Manage the state of the af.
        choices: ['present','absent']
        default: present
'''

EXAMPLES = '''
- name: Vrf af module test
  hosts: cloudengine
  connection: local
  gather_facts: no
  vars:
    cli:
      host: "{{ inventory_hostname }}"
      port: "{{ ansible_ssh_port }}"
      username: "{{ username }}"
      password: "{{ password }}"
      transport: cli

  tasks:

  - name: Config vpna, set address family is ipv4
    community.network.ce_vrf_af:
      vrf: vpna
      vrf_aftype: v4
      state: present
      provider: "{{ cli }}"
  - name: Config vpna, delete address family is ipv4
    community.network.ce_vrf_af:
      vrf: vpna
      vrf_aftype: v4
      state: absent
      provider: "{{ cli }}"
  - name: Config vpna, set address family is ipv4,rd=1:1,set vpn_target_type=export_extcommunity,vpn_target_value=2:2
    community.network.ce_vrf_af:
      vrf: vpna
      vrf_aftype: v4
      route_distinguisher: 1:1
      vpn_target_type: export_extcommunity
      vpn_target_value: 2:2
      vpn_target_state: present
      state: present
      provider: "{{ cli }}"
  - name: Config vpna, set address family is ipv4,rd=1:1,delete vpn_target_type=export_extcommunity,vpn_target_value=2:2
    community.network.ce_vrf_af:
      vrf: vpna
      vrf_aftype: v4
      route_distinguisher: 1:1
      vpn_target_type: export_extcommunity
      vpn_target_value: 2:2
      vpn_target_state: absent
      state: present
      provider: "{{ cli }}"
'''
RETURN = '''
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"vrf": "vpna",
             "vrf_aftype": "v4",
             "state": "present",
             "vpn_targe_state":"absent",
             "evpn": "none",
             "vpn_target_type": "none",
             "vpn_target_value": "none"}
existing:
    description: k/v pairs of existing switchport
    returned: always
    type: dict
    sample:  {
        "route_distinguisher": [
            "1:1",
            "2:2"
        ],
        "vpn_target_type": [],
        "vpn_target_value": [],
        "vrf": "vpna",
        "vrf_aftype": [
            "ipv4uni",
            "ipv6uni"
        ]
    }
end_state:
    description: k/v pairs of switchport after module execution
    returned: always
    type: dict
    sample:  {
        "route_distinguisher": [
            "1:1",
            "2:2"
        ],
        "vpn_target_type": [
            "import_extcommunity",
            "3:3"
        ],
        "vpn_target_value": [],
        "vrf": "vpna",
        "vrf_aftype": [
            "ipv4uni",
            "ipv6uni"
        ]
    }
updates:
    description: command list sent to the device
    returned: always
    type: list
    sample: [
        "ip vpn-instance vpna",
        "vpn-target 3:3 import_extcommunity"
    ]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
'''


import re
from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config, ce_argument_spec

CE_NC_GET_VRF = """
<filter type="subtree">
      <l3vpn xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <l3vpncomm>
          <l3vpnInstances>
            <l3vpnInstance>
              <vrfName></vrfName>
              <vrfDescription></vrfDescription>
            </l3vpnInstance>
          </l3vpnInstances>
        </l3vpncomm>
      </l3vpn>
    </filter>
"""

CE_NC_GET_VRF_AF = """
<filter type="subtree">
  <l3vpn xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <l3vpncomm>
      <l3vpnInstances>
        <l3vpnInstance>
          <vrfName>%s</vrfName>
          <vpnInstAFs>
            <vpnInstAF>
              <afType></afType>
              <vrfRD></vrfRD>%s
            </vpnInstAF>
          </vpnInstAFs>
        </l3vpnInstance>
      </l3vpnInstances>
    </l3vpncomm>
  </l3vpn>
</filter>
"""

CE_NC_DELETE_VRF_AF = """
<l3vpn xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
  <l3vpncomm>
    <l3vpnInstances>
      <l3vpnInstance>
        <vrfName>%s</vrfName>
        <vpnInstAFs>
          <vpnInstAF operation="delete">
            <afType>%s</afType>
          </vpnInstAF>
        </vpnInstAFs>
      </l3vpnInstance>
  </l3vpnInstances>
  </l3vpncomm>
</l3vpn>
"""

CE_NC_CREATE_VRF_AF = """
<l3vpn xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
  <l3vpncomm>
    <l3vpnInstances>
      <l3vpnInstance>
        <vrfName>%s</vrfName>
        <vpnInstAFs>
          <vpnInstAF operation="merge">
            <afType>%s</afType>
            <vrfRD>%s</vrfRD>%s
          </vpnInstAF>
        </vpnInstAFs>
      </l3vpnInstance>
    </l3vpnInstances>
  </l3vpncomm></l3vpn>
"""
CE_NC_CREATE_VRF_TARGET = """
<vpnTargets>
  <vpnTarget operation="merge">
    <vrfRTType>%s</vrfRTType>
    <vrfRTValue>%s</vrfRTValue>
  </vpnTarget>
</vpnTargets>
"""

CE_NC_DELETE_VRF_TARGET = """
<vpnTargets>
  <vpnTarget operation="delete">
    <vrfRTType>%s</vrfRTType>
    <vrfRTValue>%s</vrfRTValue>
  </vpnTarget>
</vpnTargets>
"""

CE_NC_GET_VRF_TARGET = """
<vpnTargets>
  <vpnTarget>
    <vrfRTValue></vrfRTValue>
    <vrfRTType></vrfRTType>
  </vpnTarget>
</vpnTargets>
"""

CE_NC_CREATE_EXTEND_VRF_TARGET = """
<exVpnTargets>
  <exVpnTarget operation="merge">
    <vrfRTType>%s</vrfRTType>
    <vrfRTValue>%s</vrfRTValue>
    <extAddrFamily>evpn</extAddrFamily>
  </exVpnTarget>
</exVpnTargets>
"""

CE_NC_DELETE_EXTEND_VRF_TARGET = """
<exVpnTargets>
  <exVpnTarget operation="delete">
    <vrfRTType>%s</vrfRTType>
    <vrfRTValue>%s</vrfRTValue>
    <extAddrFamily>evpn</extAddrFamily>
  </exVpnTarget>
</exVpnTargets>
"""

CE_NC_GET_EXTEND_VRF_TARGET = """
<exVpnTargets>
  <exVpnTarget>
    <vrfRTType></vrfRTType>
    <vrfRTValue></vrfRTValue>
    <extAddrFamily></extAddrFamily>
  </exVpnTarget>
</exVpnTargets>
"""


def build_config_xml(xmlstr):
    """build_config_xml"""

    return '<config> ' + xmlstr + ' </config>'


def is_valid_value(vrf_targe_value):
    """check if the vrf target value is valid"""

    each_num = None
    if len(vrf_targe_value) > 21 or len(vrf_targe_value) < 3:
        return False
    if vrf_targe_value.find(':') == -1:
        return False
    elif vrf_targe_value == '0:0':
        return False
    elif vrf_targe_value == '0.0:0':
        return False
    else:
        value_list = vrf_targe_value.split(':')
        if value_list[0].find('.') != -1:
            if not value_list[1].isdigit():
                return False
            if int(value_list[1]) > 65535:
                return False
            value = value_list[0].split('.')
            if len(value) == 4:
                for each_num in value:
                    if not each_num.isdigit():
                        return False
                if int(each_num) > 255:
                    return False
                return True
            elif len(value) == 2:
                for each_num in value:
                    if not each_num.isdigit():
                        return False
                if int(each_num) > 65535:
                    return False
                return True
            else:
                return False
        elif not value_list[0].isdigit():
            return False
        elif not value_list[1].isdigit():
            return False
        elif int(value_list[0]) < 65536 and int(value_list[1]) < 4294967296:
            return True
        elif int(value_list[0]) > 65535 and int(value_list[0]) < 4294967296:
            return bool(int(value_list[1]) < 65536)
        else:
            return False


class VrfAf(object):
    """manage the vrf address family and export/import target"""

    def __init__(self, argument_spec, ):
        self.spec = argument_spec
        self.module = None
        self.init_module()

        # vpn instance info
        self.vrf = self.module.params['vrf']
        self.vrf_aftype = self.module.params['vrf_aftype']
        if self.vrf_aftype == 'v4':
            self.vrf_aftype = 'ipv4uni'
        else:
            self.vrf_aftype = 'ipv6uni'
        self.route_distinguisher = self.module.params['route_distinguisher']
        self.evpn = self.module.params['evpn']
        self.vpn_target_type = self.module.params['vpn_target_type']
        self.vpn_target_value = self.module.params['vpn_target_value']
        self.vpn_target_state = self.module.params['vpn_target_state']
        self.state = self.module.params['state']

        # state
        self.changed = False
        self.updates_cmd = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()

        self.vpn_target_changed = False
        self.vrf_af_type_changed = False
        self.vrf_rd_changed = False
        self.vrf_af_info = dict()

    def init_module(self):
        """init_module"""

        self.module = AnsibleModule(
            argument_spec=self.spec, supports_check_mode=True)

    def check_response(self, xml_str, xml_name):
        """Check if response message is already succeed."""

        if "<ok/>" not in xml_str:
            self.module.fail_json(msg='Error: %s failed.' % xml_name)

    def is_vrf_af_exist(self):
        """is vrf address family exist"""

        if not self.vrf_af_info:
            return False

        for vrf_af_ele in self.vrf_af_info["vpnInstAF"]:
            if vrf_af_ele["afType"] == self.vrf_aftype:
                return True
            else:
                continue
        return False

    def get_exist_rd(self):
        """get exist route distinguisher """

        if not self.vrf_af_info:
            return None

        for vrf_af_ele in self.vrf_af_info["vpnInstAF"]:
            if vrf_af_ele["afType"] == self.vrf_aftype:
                if vrf_af_ele["vrfRD"] is None:
                    return None
                else:
                    return vrf_af_ele["vrfRD"]
            else:
                continue
        return None

    def is_vrf_rd_exist(self):
        """is vrf route distinguisher exist"""

        if not self.vrf_af_info:
            return False

        for vrf_af_ele in self.vrf_af_info["vpnInstAF"]:
            if vrf_af_ele["afType"] == self.vrf_aftype:
                if vrf_af_ele["vrfRD"] is None:
                    return False
                if self.route_distinguisher is not None:
                    return bool(vrf_af_ele["vrfRD"] == self.route_distinguisher)
                else:
                    return True
            else:
                continue
        return False

    def is_vrf_rt_exist(self):
        """is vpn target exist"""

        if not self.vrf_af_info:
            return False

        for vrf_af_ele in self.vrf_af_info["vpnInstAF"]:
            if vrf_af_ele["afType"] == self.vrf_aftype:
                if self.evpn is False:
                    if not vrf_af_ele.get("vpnTargets"):
                        return False
                    for vpn_target in vrf_af_ele.get("vpnTargets"):
                        if vpn_target["vrfRTType"] == self.vpn_target_type \
                                and vpn_target["vrfRTValue"] == self.vpn_target_value:
                            return True
                        else:
                            continue
                else:
                    if not vrf_af_ele.get("evpnTargets"):
                        return False
                    for evpn_target in vrf_af_ele.get("evpnTargets"):
                        if evpn_target["vrfRTType"] == self.vpn_target_type \
                                and evpn_target["vrfRTValue"] == self.vpn_target_value:
                            return True
                        else:
                            continue
            else:
                continue
        return False

    def set_update_cmd(self):
        """ set update command"""
        if not self.changed:
            return
        if self.vpn_target_type:
            if self.vpn_target_type == "export_extcommunity":
                vpn_target_type = "export-extcommunity"
            else:
                vpn_target_type = "import-extcommunity"
        if self.state == "present":
            self.updates_cmd.append('ip vpn-instance %s' % (self.vrf))
            if self.vrf_aftype == 'ipv4uni':
                self.updates_cmd.append('ipv4-family')
            elif self.vrf_aftype == 'ipv6uni':
                self.updates_cmd.append('ipv6-family')
            if self.route_distinguisher:
                if not self.is_vrf_rd_exist():
                    self.updates_cmd.append(
                        'route-distinguisher %s' % self.route_distinguisher)
            else:
                if self.get_exist_rd() is not None:
                    self.updates_cmd.append(
                        'undo route-distinguisher %s' % self.get_exist_rd())
            if self.vpn_target_state == "present":
                if not self.is_vrf_rt_exist():
                    if self.evpn is False:
                        self.updates_cmd.append(
                            'vpn-target %s %s' % (self.vpn_target_value, vpn_target_type))
                    else:
                        self.updates_cmd.append(
                            'vpn-target %s %s evpn' % (self.vpn_target_value, vpn_target_type))
            elif self.vpn_target_state == "absent":
                if self.is_vrf_rt_exist():
                    if self.evpn is False:
                        self.updates_cmd.append(
                            'undo vpn-target %s %s' % (self.vpn_target_value, vpn_target_type))
                    else:
                        self.updates_cmd.append(
                            'undo vpn-target %s %s evpn' % (self.vpn_target_value, vpn_target_type))
        else:
            self.updates_cmd.append('ip vpn-instance %s' % (self.vrf))
            if self.vrf_aftype == 'ipv4uni':
                self.updates_cmd.append('undo ipv4-family')
            elif self.vrf_aftype == 'ipv6uni':
                self.updates_cmd.append('undo ipv6-family')

    def get_vrf(self):
        """ check if vrf is need to change"""

        getxmlstr = CE_NC_GET_VRF
        xmlstr_new_1 = (self.vrf.lower())

        xml_str = get_nc_config(self.module, getxmlstr)
        re_find_1 = re.findall(
            r'.*<vrfname>(.*)</vrfname>.*', xml_str.lower())

        if re_find_1 is None:
            return False

        return xmlstr_new_1 in re_find_1

    def get_vrf_af(self):
        """ check if vrf is need to change"""

        self.vrf_af_info["vpnInstAF"] = list()
        if self.evpn is True:
            getxmlstr = CE_NC_GET_VRF_AF % (
                self.vrf, CE_NC_GET_EXTEND_VRF_TARGET)
        else:
            getxmlstr = CE_NC_GET_VRF_AF % (self.vrf, CE_NC_GET_VRF_TARGET)

        xml_str = get_nc_config(self.module, getxmlstr)

        if 'data/' in xml_str:
            return self.state == 'present'
        xml_str = xml_str.replace('\r', '').replace('\n', '').\
            replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
            replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

        root = ElementTree.fromstring(xml_str)

        # get the vpn address family and RD text
        vrf_addr_types = root.findall(
            "l3vpn/l3vpncomm/l3vpnInstances/l3vpnInstance/vpnInstAFs/vpnInstAF")
        if vrf_addr_types:
            for vrf_addr_type in vrf_addr_types:
                vrf_af_info = dict()
                for vrf_addr_type_ele in vrf_addr_type:
                    if vrf_addr_type_ele.tag in ["vrfName", "afType", "vrfRD"]:
                        vrf_af_info[vrf_addr_type_ele.tag] = vrf_addr_type_ele.text
                    if vrf_addr_type_ele.tag == 'vpnTargets':
                        vrf_af_info["vpnTargets"] = list()
                        for rtargets in vrf_addr_type_ele:
                            rt_dict = dict()
                            for rtarget in rtargets:
                                if rtarget.tag in ["vrfRTValue", "vrfRTType"]:
                                    rt_dict[rtarget.tag] = rtarget.text
                            vrf_af_info["vpnTargets"].append(rt_dict)
                    if vrf_addr_type_ele.tag == 'exVpnTargets':
                        vrf_af_info["evpnTargets"] = list()
                        for rtargets in vrf_addr_type_ele:
                            rt_dict = dict()
                            for rtarget in rtargets:
                                if rtarget.tag in ["vrfRTValue", "vrfRTType"]:
                                    rt_dict[rtarget.tag] = rtarget.text
                            vrf_af_info["evpnTargets"].append(rt_dict)
                self.vrf_af_info["vpnInstAF"].append(vrf_af_info)

    def check_params(self):
        """Check all input params"""

        # vrf and description check
        if self.vrf == '_public_':
            self.module.fail_json(
                msg='Error: The vrf name _public_ is reserved.')
        if not self.get_vrf():
            self.module.fail_json(
                msg='Error: The vrf name do not exist.')
        if self.state == 'present':
            if self.route_distinguisher:
                if not is_valid_value(self.route_distinguisher):
                    self.module.fail_json(msg='Error:The vrf route distinguisher length must between 3 ~ 21,'
                                          'i.e. X.X.X.X:number<0-65535> or number<0-65535>:number<0-4294967295>'
                                          'or number<0-65535>.number<0-65535>:number<0-65535>'
                                          'or number<65536-4294967295>:number<0-65535>'
                                          ' but not be 0:0 or 0.0:0.')
            if not self.vpn_target_state:
                if self.vpn_target_value or self.vpn_target_type:
                    self.module.fail_json(
                        msg='Error: The vpn target state should be exist.')
            if self.vpn_target_state:
                if not self.vpn_target_value or not self.vpn_target_type:
                    self.module.fail_json(
                        msg='Error: The vpn target value and type should be exist.')
            if self.vpn_target_value:
                if not is_valid_value(self.vpn_target_value):
                    self.module.fail_json(msg='Error:The vrf target value length must between 3 ~ 21,'
                                          'i.e. X.X.X.X:number<0-65535> or number<0-65535>:number<0-4294967295>'
                                          'or number<0-65535>.number<0-65535>:number<0-65535>'
                                          'or number<65536-4294967295>:number<0-65535>'
                                          ' but not be 0:0 or 0.0:0.')

    def operate_vrf_af(self):
        """config/delete vrf"""

        vrf_target_operate = ''
        if self.route_distinguisher is None:
            route_d = ''
        else:
            route_d = self.route_distinguisher

        if self.state == 'present':
            if self.vrf_aftype:
                if self.is_vrf_af_exist():
                    self.vrf_af_type_changed = False
                else:
                    self.vrf_af_type_changed = True
                    configxmlstr = CE_NC_CREATE_VRF_AF % (
                        self.vrf, self.vrf_aftype, route_d, vrf_target_operate)
            else:
                self.vrf_af_type_changed = bool(self.is_vrf_af_exist())

            if self.vpn_target_state == 'present':
                if self.evpn is False and not self.is_vrf_rt_exist():
                    vrf_target_operate = CE_NC_CREATE_VRF_TARGET % (
                        self.vpn_target_type, self.vpn_target_value)
                    configxmlstr = CE_NC_CREATE_VRF_AF % (
                        self.vrf, self.vrf_aftype, route_d, vrf_target_operate)
                    self.vpn_target_changed = True
                if self.evpn is True and not self.is_vrf_rt_exist():
                    vrf_target_operate = CE_NC_CREATE_EXTEND_VRF_TARGET % (
                        self.vpn_target_type, self.vpn_target_value)
                    configxmlstr = CE_NC_CREATE_VRF_AF % (
                        self.vrf, self.vrf_aftype, route_d, vrf_target_operate)
                    self.vpn_target_changed = True
            elif self.vpn_target_state == 'absent':
                if self.evpn is False and self.is_vrf_rt_exist():
                    vrf_target_operate = CE_NC_DELETE_VRF_TARGET % (
                        self.vpn_target_type, self.vpn_target_value)
                    configxmlstr = CE_NC_CREATE_VRF_AF % (
                        self.vrf, self.vrf_aftype, route_d, vrf_target_operate)
                    self.vpn_target_changed = True
                if self.evpn is True and self.is_vrf_rt_exist():
                    vrf_target_operate = CE_NC_DELETE_EXTEND_VRF_TARGET % (
                        self.vpn_target_type, self.vpn_target_value)
                    configxmlstr = CE_NC_CREATE_VRF_AF % (
                        self.vrf, self.vrf_aftype, route_d, vrf_target_operate)
                    self.vpn_target_changed = True
            else:
                if self.route_distinguisher:
                    if not self.is_vrf_rd_exist():
                        configxmlstr = CE_NC_CREATE_VRF_AF % (
                            self.vrf, self.vrf_aftype, route_d, vrf_target_operate)
                        self.vrf_rd_changed = True
                    else:
                        self.vrf_rd_changed = False
                else:
                    if self.is_vrf_rd_exist():
                        configxmlstr = CE_NC_CREATE_VRF_AF % (
                            self.vrf, self.vrf_aftype, route_d, vrf_target_operate)
                        self.vrf_rd_changed = True
                    else:
                        self.vrf_rd_changed = False
            if not self.vrf_rd_changed and not self.vrf_af_type_changed and not self.vpn_target_changed:
                self.changed = False
            else:
                self.changed = True
        else:
            if self.is_vrf_af_exist():
                configxmlstr = CE_NC_DELETE_VRF_AF % (
                    self.vrf, self.vrf_aftype)
                self.changed = True
            else:
                self.changed = False

        if not self.changed:
            return

        conf_str = build_config_xml(configxmlstr)

        recv_xml = set_nc_config(self.module, conf_str)
        self.check_response(recv_xml, "OPERATE_VRF_AF")

    def get_proposed(self):
        """get_proposed"""

        if self.state == 'present':
            self.proposed['vrf'] = self.vrf
            if self.vrf_aftype is None:
                self.proposed['vrf_aftype'] = 'ipv4uni'
            else:
                self.proposed['vrf_aftype'] = self.vrf_aftype
            if self.route_distinguisher is not None:
                self.proposed['route_distinguisher'] = self.route_distinguisher
            else:
                self.proposed['route_distinguisher'] = list()
            if self.vpn_target_state == 'present':
                self.proposed['evpn'] = self.evpn
                self.proposed['vpn_target_type'] = self.vpn_target_type
                self.proposed['vpn_target_value'] = self.vpn_target_value
            else:
                self.proposed['vpn_target_type'] = list()
                self.proposed['vpn_target_value'] = list()
        else:
            self.proposed = dict()
            self.proposed['state'] = self.state
            self.proposed['vrf'] = self.vrf
            self.proposed['vrf_aftype'] = list()
            self.proposed['route_distinguisher'] = list()
            self.proposed['vpn_target_value'] = list()
            self.proposed['vpn_target_type'] = list()

    def get_existing(self):
        """get_existing"""

        self.get_vrf_af()
        self.existing['vrf'] = self.vrf
        self.existing['vrf_aftype'] = list()
        self.existing['route_distinguisher'] = list()
        self.existing['vpn_target_value'] = list()
        self.existing['vpn_target_type'] = list()
        self.existing['evpn_target_value'] = list()
        self.existing['evpn_target_type'] = list()
        if self.vrf_af_info["vpnInstAF"] is None:
            return
        for vrf_af_ele in self.vrf_af_info["vpnInstAF"]:
            self.existing['vrf_aftype'].append(vrf_af_ele["afType"])
            self.existing['route_distinguisher'].append(
                vrf_af_ele["vrfRD"])
            if vrf_af_ele.get("vpnTargets"):
                for vpn_target in vrf_af_ele.get("vpnTargets"):
                    self.existing['vpn_target_type'].append(
                        vpn_target["vrfRTType"])
                    self.existing['vpn_target_value'].append(
                        vpn_target["vrfRTValue"])
            if vrf_af_ele.get("evpnTargets"):
                for evpn_target in vrf_af_ele.get("evpnTargets"):
                    self.existing['evpn_target_type'].append(
                        evpn_target["vrfRTType"])
                    self.existing['evpn_target_value'].append(
                        evpn_target["vrfRTValue"])

    def get_end_state(self):
        """get_end_state"""

        self.get_vrf_af()
        self.end_state['vrf'] = self.vrf
        self.end_state['vrf_aftype'] = list()
        self.end_state['route_distinguisher'] = list()
        self.end_state['vpn_target_value'] = list()
        self.end_state['vpn_target_type'] = list()
        self.end_state['evpn_target_value'] = list()
        self.end_state['evpn_target_type'] = list()
        if self.vrf_af_info["vpnInstAF"] is None:
            return
        for vrf_af_ele in self.vrf_af_info["vpnInstAF"]:
            self.end_state['vrf_aftype'].append(vrf_af_ele["afType"])
            self.end_state['route_distinguisher'].append(vrf_af_ele["vrfRD"])
            if vrf_af_ele.get("vpnTargets"):
                for vpn_target in vrf_af_ele.get("vpnTargets"):
                    self.end_state['vpn_target_type'].append(
                        vpn_target["vrfRTType"])
                    self.end_state['vpn_target_value'].append(
                        vpn_target["vrfRTValue"])
            if vrf_af_ele.get("evpnTargets"):
                for evpn_target in vrf_af_ele.get("evpnTargets"):
                    self.end_state['evpn_target_type'].append(
                        evpn_target["vrfRTType"])
                    self.end_state['evpn_target_value'].append(
                        evpn_target["vrfRTValue"])

    def work(self):
        """worker"""

        self.check_params()
        self.get_existing()
        self.get_proposed()
        self.operate_vrf_af()
        self.set_update_cmd()
        self.get_end_state()
        self.results['changed'] = self.changed
        self.results['proposed'] = self.proposed
        self.results['existing'] = self.existing
        self.results['end_state'] = self.end_state
        if self.changed:
            self.results['updates'] = self.updates_cmd
        else:
            self.results['updates'] = list()

        self.module.exit_json(**self.results)


def main():
    """main"""

    argument_spec = dict(
        vrf=dict(required=True, type='str'),
        vrf_aftype=dict(choices=['v4', 'v6'],
                        default='v4', required=False),
        route_distinguisher=dict(required=False, type='str'),
        evpn=dict(type='bool', default=False),
        vpn_target_type=dict(
            choices=['export_extcommunity', 'import_extcommunity'], required=False),
        vpn_target_value=dict(required=False, type='str'),
        vpn_target_state=dict(choices=['absent', 'present'], required=False),
        state=dict(choices=['absent', 'present'],
                   default='present', required=False),
    )
    argument_spec.update(ce_argument_spec)
    interface = VrfAf(argument_spec)
    interface.work()


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr_acls class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type


from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.utils.utils import (
    flatten_dict,
    prefix_to_address_wildcard,
    is_ipv4_address,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    to_list,
    search_obj_in_list,
    dict_diff,
    remove_empties,
)
from ansible.module_utils.six import iteritems
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import (
    Facts,
)


class Acls(ConfigBase):
    """
    The iosxr_acls class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["acls"]

    def __init__(self, module):
        super(Acls, self).__init__(module)

    def get_acls_facts(self, data=None):
        """ Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset, self.gather_network_resources, data=data
        )
        acls_facts = facts["ansible_network_resources"].get("acls")
        if not acls_facts:
            return []
        return acls_facts

    def execute_module(self):
        """ Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        warnings = list()
        commands = list()

        if self.state in self.ACTION_STATES:
            existing_acls_facts = self.get_acls_facts()
        else:
            existing_acls_facts = []

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_acls_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_acls_facts = self.get_acls_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed"
                )
            result["parsed"] = self.get_acls_facts(data=running_config)

        if self.state in self.ACTION_STATES:
            result["before"] = existing_acls_facts
            if result["changed"]:
                result["after"] = changed_acls_facts

        elif self.state == "gathered":
            result["gathered"] = changed_acls_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_acls_facts):
        """ Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_acls_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """ Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        commands = []

        if (
            state in ("overridden", "merged", "replaced", "rendered")
            and not want
        ):
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    state
                )
            )

        if state == "overridden":
            commands.extend(self._state_overridden(want, have))

        elif state == "deleted":
            commands.extend(self._state_deleted(want, have))

        else:
            # Instead of passing entire want and have
            # list of dictionaries to the respective
            # _state_* methods we are passing the want
            # and have dictionaries per AFI
            for item in want:
                afi = item["afi"]
                obj_in_have = search_obj_in_list(afi, have, key="afi")

                if state == "merged" or self.state == "rendered":
                    commands.extend(
                        self._state_merged(remove_empties(item), obj_in_have)
                    )

                elif state == "replaced":
                    commands.extend(
                        self._state_replaced(remove_empties(item), obj_in_have)
                    )

        return commands

    def _state_replaced(self, want, have):
        """ The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []

        for want_acl in want["acls"]:
            have_acl = search_obj_in_list(want_acl["name"], have["acls"]) or {}
            acl_updates = []

            for have_ace in have_acl.get("aces", []):
                want_ace = (
                    search_obj_in_list(
                        have_ace["sequence"], want_acl["aces"], key="sequence"
                    )
                    or {}
                )
                if not want_ace:
                    acl_updates.append("no {0}".format(have_ace["sequence"]))

            for want_ace in want_acl.get("aces", []):
                have_ace = (
                    search_obj_in_list(
                        want_ace.get("sequence"),
                        have_acl.get("aces", []),
                        key="sequence",
                    )
                    or {}
                )
                set_cmd = self._set_commands(want_ace, have_ace)
                if set_cmd:
                    acl_updates.append(set_cmd)

            if acl_updates:
                acl_updates.insert(
                    0,
                    "{0} access-list {1}".format(
                        want["afi"], want_acl["name"]
                    ),
                )
                commands.extend(acl_updates)

        return commands

    def _state_overridden(self, want, have):
        """ The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []

        # Remove extraneous AFI that are present in config but not
        # specified in `want`
        for have_afi in have:
            want_afi = (
                search_obj_in_list(have_afi["afi"], want, key="afi") or {}
            )
            if not want_afi:
                for acl in have_afi.get("acls", []):
                    commands.append(
                        "no {0} access-list {1}".format(
                            have_afi["afi"], acl["name"]
                        )
                    )

        # First we remove the extraneous ACLs from the AFIs that
        # are present both in `want` and in `have` and then
        # we call `_state_replaced` to update the ACEs within those ACLs
        for want_afi in want:
            want_afi = remove_empties(want_afi)
            have_afi = (
                search_obj_in_list(want_afi["afi"], have, key="afi") or {}
            )
            if have_afi:
                for have_acl in have_afi.get("acls", []):
                    want_acl = (
                        search_obj_in_list(
                            have_acl["name"], want_afi.get("acls", [])
                        )
                        or {}
                    )
                    if not want_acl:
                        commands.append(
                            "no {0} access-list {1}".format(
                                have_afi["afi"], have_acl["name"]
                            )
                        )

            commands.extend(self._state_replaced(want_afi, have_afi))

        return commands

    def _state_merged(self, want, have):
        """ The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []
        if not have:
            have = {}

        for want_acl in want["acls"]:
            have_acl = (
                search_obj_in_list(want_acl["name"], have.get("acls", {}))
                or {}
            )

            acl_updates = []
            for want_ace in want_acl["aces"]:
                have_ace = (
                    search_obj_in_list(
                        want_ace.get("sequence"),
                        have_acl.get("aces", []),
                        key="sequence",
                    )
                    or {}
                )
                set_cmd = self._set_commands(want_ace, have_ace)
                if set_cmd:
                    acl_updates.append(set_cmd)

            if acl_updates:
                acl_updates.insert(
                    0,
                    "{0} access-list {1}".format(
                        want["afi"], want_acl["name"]
                    ),
                )
                commands.extend(acl_updates)

        return commands

    def _state_deleted(self, want, have):
        """ The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []

        if not want:
            want = [{"afi": "ipv4"}, {"afi": "ipv6"}]

        for item in want:
            item = remove_empties(item)
            have_item = search_obj_in_list(item["afi"], have, key="afi") or {}
            if "acls" not in item:
                if have_item:
                    for acl in have_item["acls"]:
                        commands.append(
                            "no {0} access-list {1}".format(
                                have_item["afi"], acl["name"]
                            )
                        )
            else:
                for want_acl in item["acls"]:
                    have_acl = (
                        search_obj_in_list(
                            want_acl["name"], have_item.get("acls", [])
                        )
                        or {}
                    )
                    if have_acl:
                        commands.append(
                            "no {0} access-list {1}".format(
                                have_item["afi"], have_acl["name"]
                            )
                        )

        return commands

    def _compute_commands(self, want_ace):
        """This command creates an ACE line from an ACE dictionary

        :rtype: A string
        :returns: An ACE generated from a structured ACE dictionary
        """

        def __compute_src_dest(dir_dict):
            cmd = ""
            if "any" in dir_dict:
                cmd += "any "
            elif "host" in dir_dict:
                cmd += "host {0} ".format(dir_dict["host"])
            elif "prefix" in dir_dict:
                cmd += "{0} ".format(dir_dict["prefix"])
            else:
                cmd += "{0} {1} ".format(
                    dir_dict["address"], dir_dict["wildcard_bits"]
                )

            if "port_protocol" in dir_dict:
                protocol_range = dir_dict["port_protocol"].get("range")
                if protocol_range:
                    cmd += "range {0} {1} ".format(
                        protocol_range["start"], protocol_range["end"]
                    )
                else:
                    for key, value in iteritems(dir_dict["port_protocol"]):
                        cmd += "{0} {1} ".format(key, value)

            return cmd

        def __compute_protocol_options(protocol_dict):
            cmd = ""
            for value in protocol_options.values():
                for subkey, subvalue in iteritems(value):
                    if subvalue:
                        cmd += "{0} ".format(subkey.replace("_", "-"))
            return cmd

        def __compute_match_options(want_ace):
            cmd = ""

            if "precedence" in want_ace:
                cmd += "precedence {0} ".format(want_ace["precedence"])

            for x in ["dscp", "packet_length", "ttl"]:
                if x in want_ace:
                    opt_range = want_ace[x].get("range")
                    if opt_range:
                        cmd += "{0} range {1} {2} ".format(
                            x.replace("_", "-"),
                            opt_range["start"],
                            opt_range["end"],
                        )
                    else:
                        for key, value in iteritems(want_ace[x]):
                            cmd += "{0} {1} {2} ".format(
                                x.replace("_", "-"), key, value
                            )

            for x in (
                "authen",
                "capture",
                "fragments",
                "routing",
                "log",
                "log_input",
                "icmp_off",
                "destopts",
                "hop_by_hop",
            ):
                if x in want_ace:
                    cmd += "{0} ".format(x.replace("_", "-"))

            return cmd

        cmd = ""
        if "sequence" in want_ace:
            cmd += "{0} ".format(want_ace["sequence"])

        if "remark" in want_ace:
            cmd += "remark {0}".format(want_ace["remark"])

        elif "line" in want_ace:
            cmd += want_ace["line"]

        else:
            cmd += "{0} ".format(want_ace["grant"])
            if "protocol" in want_ace:
                cmd += "{0} ".format(want_ace["protocol"])

            cmd += __compute_src_dest(want_ace["source"])
            cmd += __compute_src_dest(want_ace["destination"])

            protocol_options = want_ace.get("protocol_options", {})
            if protocol_options:
                cmd += __compute_protocol_options(protocol_options)

            cmd += __compute_match_options(want_ace)

        return cmd.strip()

    def _set_commands(self, want_ace, have_ace):
        """A helped method that checks if there is
           a delta between the `have_ace` and `want_ace`.
           If there is a delta then it calls `_compute_commands`
           to create the ACE line.

        :rtype: A string
        :returns: An ACE generated from a structured ACE dictionary
                  via a call to `_compute_commands`
        """

        if "line" in want_ace:
            if want_ace["line"] != have_ace.get("line"):
                return self._compute_commands(want_ace)

        else:
            if ("prefix" in want_ace.get("source", {})) or (
                "prefix" in want_ace.get("destination", {})
            ):
                self._prepare_for_diff(want_ace)

            protocol_opt_delta = {}
            delta = dict_diff(have_ace, want_ace)

            # `dict_diff` doesn't work properly for `protocol_options` diff,
            # so we need to handle it separately
            if want_ace.get("protocol_options", {}):
                protocol_opt_delta = set(
                    flatten_dict(have_ace.get("protocol_options", {}))
                ) ^ set(flatten_dict(want_ace.get("protocol_options", {})))

            if delta or protocol_opt_delta:
                want_ace = self._dict_merge(have_ace, want_ace)
                return self._compute_commands(want_ace)

    def _prepare_for_diff(self, ace):
        """This method prepares the want ace dict
           for diff calculation against the have ace dict.

        :param ace: The want ace to prepare for diff calculation
        """
        # Convert prefixes to "address wildcard bits" format for IPv4 addresses
        # Not valid for IPv6 addresses because those can only be specified as prefixes
        # and are always rendered in running-config as prefixes too
        for x in ["source", "destination"]:
            prefix = ace.get(x, {}).get("prefix")
            if prefix and is_ipv4_address(prefix):
                del ace[x]["prefix"]
                ace[x]["address"], ace[x][
                    "wildcard_bits"
                ] = prefix_to_address_wildcard(prefix)

    def _dict_merge(self, have_ace, want_ace):
        for x in want_ace:
            have_ace[x] = want_ace[x]
        return have_ace



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

__metaclass__ = type

# Copyright: (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = """
module: eos_logging
author: Trishna Guha (@trishnaguha)
short_description: Manage logging on network devices
description:
- This module provides declarative management of logging on Arista Eos devices.
version_added: 1.0.0
notes:
- Tested against EOS 4.15
options:
  dest:
    description:
    - Destination of the logs.
    choices:
    - "on"
    - host
    - console
    - monitor
    - buffered
    type: str
  name:
    description:
    - The hostname or IP address of the destination.
    - Required when I(dest=host).
    type: str
  size:
    description:
    - Size of buffer. The acceptable value is in range from 10 to 2147483647 bytes.
    type: int
  facility:
    description:
    - Set logging facility.
    type: str
  level:
    description:
    - Set logging severity levels.
    choices:
    - emergencies
    - alerts
    - critical
    - errors
    - warnings
    - notifications
    - informational
    - debugging
    type: str
  aggregate:
    description: List of logging definitions.
    type: list
    elements: dict
    suboptions:
      dest:
        description:
        - Destination of the logs.
        choices:
        - "on"
        - host
        - console
        - monitor
        - buffered
        type: str
      name:
        description:
        - The hostname or IP address of the destination.
        - Required when I(dest=host).
        type: str
      size:
        description:
        - Size of buffer. The acceptable value is in range from 10 to 2147483647 bytes.
        type: int
      facility:
        description:
        - Set logging facility.
        type: str
      level:
        description:
        - Set logging severity levels.
        choices:
        - emergencies
        - alerts
        - critical
        - errors
        - warnings
        - notifications
        - informational
        - debugging
        type: str
      state:
        description:
        - State of the logging configuration.
        default: present
        type: str
        choices:
        - present
        - absent
  state:
    description:
    - State of the logging configuration.
    default: present
    type: str
    choices:
    - present
    - absent
extends_documentation_fragment:
- arista.eos.eos
"""

EXAMPLES = """
- name: configure host logging
  arista.eos.eos_logging:
    dest: host
    name: 172.16.0.1
    state: present

- name: remove host logging configuration
  arista.eos.eos_logging:
    dest: host
    name: 172.16.0.1
    state: absent

- name: configure console logging level and facility
  arista.eos.eos_logging:
    dest: console
    facility: local7
    level: debugging
    state: present

- name: enable logging to all
  arista.eos.eos_logging:
    dest: on

- name: configure buffer size
  arista.eos.eos_logging:
    dest: buffered
    size: 5000

- name: Configure logging using aggregate
  arista.eos.eos_logging:
    aggregate:
    - {dest: console, level: warnings}
    - {dest: buffered, size: 480000}
    state: present
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - logging facility local7
    - logging host 172.16.0.1
"""

import re


from copy import deepcopy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    get_config,
    load_config,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    eos_argument_spec,
)


DEST_GROUP = ["on", "host", "console", "monitor", "buffered"]
LEVEL_GROUP = [
    "emergencies",
    "alerts",
    "critical",
    "errors",
    "warnings",
    "notifications",
    "informational",
    "debugging",
]


def validate_size(value, module):
    if value:
        if not int(10) <= value <= int(2147483647):
            module.fail_json(msg="size must be between 10 and 2147483647")
        else:
            return value


def map_obj_to_commands(updates, module):
    commands = list()
    want, have = updates

    for w in want:
        dest = w["dest"]
        name = w["name"]
        size = w["size"]
        facility = w["facility"]
        level = w["level"]
        state = w["state"]
        del w["state"]

        if state == "absent" and w in have:
            if dest:
                if dest == "host":
                    commands.append("no logging host {0}".format(name))

                elif dest in DEST_GROUP:
                    commands.append("no logging {0}".format(dest))

                else:
                    module.fail_json(
                        msg="dest must be among console, monitor, buffered, host, on"
                    )

            if facility:
                commands.append("no logging facility {0}".format(facility))

        if state == "present" and w not in have:
            if facility:
                present = False

                # Iterate over every dictionary in the 'have' list to check if
                # similar configuration for facility exists or not

                for entry in have:
                    if not entry["dest"] and entry["facility"] == facility:
                        present = True

                if not present:
                    commands.append("logging facility {0}".format(facility))

            if dest == "host":
                commands.append("logging host {0}".format(name))

            elif dest == "on":
                commands.append("logging on")

            elif dest == "buffered" and size:

                present = False

                # Deals with the following two cases:
                # Case 1:       logging buffered <size> <level>
                #               logging buffered <same-size>
                #
                # Case 2:       Same buffered logging configuration
                #               already exists (i.e., both size &
                #               level are same)

                for entry in have:
                    if entry["dest"] == "buffered" and entry["size"] == size:

                        if not level or entry["level"] == level:
                            present = True

                if not present:
                    if size and level:
                        commands.append(
                            "logging buffered {0} {1}".format(size, level)
                        )
                    else:
                        commands.append("logging buffered {0}".format(size))

            else:
                if dest:
                    dest_cmd = "logging {0}".format(dest)
                    if level:
                        dest_cmd += " {0}".format(level)

                    commands.append(dest_cmd)
    return commands


def parse_facility(line):
    facility = None
    match = re.search(r"logging facility (\S+)", line, re.M)
    if match:
        facility = match.group(1)

    return facility


def parse_size(line, dest):
    size = None

    if dest == "buffered":
        match = re.search(r"logging buffered (\S+)", line, re.M)
        if match:
            try:
                int_size = int(match.group(1))
            except ValueError:
                int_size = None

            if int_size:
                if isinstance(int_size, int):
                    size = str(match.group(1))
                else:
                    size = str(10)

    return size


def parse_name(line, dest):
    name = None
    if dest == "host":
        match = re.search(r"logging host (\S+)", line, re.M)
        if match:
            name = match.group(1)

    return name


def parse_level(line, dest):
    level = None

    if dest != "host":

        # Line for buffer logging entry in running-config is of the form:
        # logging buffered <size> <level>

        if dest == "buffered":
            match = re.search(r"logging buffered (?:\d+) (\S+)", line, re.M)

        else:
            match = re.search(r"logging {0} (\S+)".format(dest), line, re.M)

        if match:
            if match.group(1) in LEVEL_GROUP:
                level = match.group(1)

    return level


def map_config_to_obj(module):
    obj = []

    data = get_config(module, flags=["section logging"])

    for line in data.split("\n"):

        match = re.search(r"logging (\S+)", line, re.M)

        if match:
            if match.group(1) in DEST_GROUP:
                dest = match.group(1)

            else:
                dest = None

            obj.append(
                {
                    "dest": dest,
                    "name": parse_name(line, dest),
                    "size": parse_size(line, dest),
                    "facility": parse_facility(line),
                    "level": parse_level(line, dest),
                }
            )

    return obj


def parse_obj(obj, module):
    if module.params["size"] is None:
        obj.append(
            {
                "dest": module.params["dest"],
                "name": module.params["name"],
                "size": module.params["size"],
                "facility": module.params["facility"],
                "level": module.params["level"],
                "state": module.params["state"],
            }
        )

    else:
        obj.append(
            {
                "dest": module.params["dest"],
                "name": module.params["name"],
                "size": str(validate_size(module.params["size"], module)),
                "facility": module.params["facility"],
                "level": module.params["level"],
                "state": module.params["state"],
            }
        )

    return obj


def map_params_to_obj(module, required_if=None):
    obj = []
    aggregate = module.params.get("aggregate")
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]

            module._check_required_if(required_if, item)
            d = item.copy()

            if d["dest"] != "host":
                d["name"] = None

            if d["dest"] == "buffered":
                if "size" in d:
                    d["size"] = str(validate_size(d["size"], module))
                elif "size" not in d:
                    d["size"] = str(10)
                else:
                    pass

            if d["dest"] != "buffered":
                d["size"] = None

            obj.append(d)

    else:
        if module.params["dest"] != "host":
            module.params["name"] = None

        if module.params["dest"] == "buffered":
            if not module.params["size"]:
                module.params["size"] = str(10)
        else:
            module.params["size"] = None

        parse_obj(obj, module)

    return obj


def main():
    """ main entry point for module execution
    """
    element_spec = dict(
        dest=dict(choices=DEST_GROUP),
        name=dict(),
        size=dict(type="int"),
        facility=dict(),
        level=dict(choices=LEVEL_GROUP),
        state=dict(default="present", choices=["present", "absent"]),
    )

    aggregate_spec = deepcopy(element_spec)

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    aggregate_spec["state"].update(default="present")
    argument_spec = dict(
        aggregate=dict(type="list", elements="dict", options=aggregate_spec)
    )

    argument_spec.update(element_spec)
    argument_spec.update(eos_argument_spec)

    required_if = [("dest", "host", ["name"])]

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    warnings = list()

    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings

    have = map_config_to_obj(module)
    want = map_params_to_obj(module, required_if=required_if)

    commands = map_obj_to_commands((want, have), module)
    result["commands"] = commands

    if commands:
        commit = not module.check_mode
        response = load_config(module, commands, commit=commit)
        if response.get("diff") and module._diff:
            result["diff"] = {"prepared": response.get("diff")}
        result["session_name"] = response.get("session")
        result["changed"] = True

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr_static_routes class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    to_list,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import (
    Facts,
)
from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    search_obj_in_list,
    remove_empties,
    dict_diff,
    dict_merge,
)


class Static_routes(ConfigBase):
    """
    The iosxr_static_routes class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["static_routes"]

    def __init__(self, module):
        super(Static_routes, self).__init__(module)

    def get_static_routes_facts(self, data=None):
        """ Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset, self.gather_network_resources, data=data
        )
        static_routes_facts = facts["ansible_network_resources"].get(
            "static_routes"
        )
        if not static_routes_facts:
            return []
        return static_routes_facts

    def execute_module(self):
        """ Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        warnings = list()
        commands = list()

        if self.state in self.ACTION_STATES:
            existing_static_routes_facts = self.get_static_routes_facts()
        else:
            existing_static_routes_facts = []

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_static_routes_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_static_routes_facts = self.get_static_routes_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed"
                )
            result["parsed"] = self.get_static_routes_facts(
                data=running_config
            )

        if self.state in self.ACTION_STATES:
            result["before"] = existing_static_routes_facts
            if result["changed"]:
                result["after"] = changed_static_routes_facts

        elif self.state == "gathered":
            result["gathered"] = changed_static_routes_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_static_routes_facts):
        """ Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_static_routes_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """ Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        commands = []

        if (
            state in ("overridden", "merged", "replaced", "rendered")
            and not want
        ):
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    state
                )
            )

        if state == "overridden":
            commands.extend(self._state_overridden(want, have))

        elif state == "deleted":
            if not want:
                if len(have) >= 1:
                    return "no router static"

            else:
                for w_item in want:
                    obj_in_have = self._find_vrf(w_item, have)
                    if obj_in_have:
                        commands.extend(
                            self._state_deleted(
                                remove_empties(w_item), obj_in_have
                            )
                        )

        else:
            for w_item in want:
                obj_in_have = self._find_vrf(w_item, have)
                if state == "merged" or self.state == "rendered":
                    commands.extend(
                        self._state_merged(remove_empties(w_item), obj_in_have)
                    )

                elif state == "replaced":
                    commands.extend(
                        self._state_replaced(
                            remove_empties(w_item), obj_in_have
                        )
                    )

        if commands:
            commands.insert(0, "router static")

        return commands

    def _state_replaced(self, want, have):
        """ The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []

        for want_afi in want.get("address_families", []):
            have_afi = (
                self.find_af_context(
                    want_afi, have.get("address_families", [])
                )
                or {}
            )
            update_commands = []
            for want_route in want_afi.get("routes", []):
                have_route = (
                    search_obj_in_list(
                        want_route["dest"],
                        have_afi.get("routes", []),
                        key="dest",
                    )
                    or {}
                )

                rotated_have_next_hops = self.rotate_next_hops(
                    have_route.get("next_hops", {})
                )
                rotated_want_next_hops = self.rotate_next_hops(
                    want_route.get("next_hops", {})
                )

                for key in rotated_have_next_hops.keys():
                    if key not in rotated_want_next_hops:
                        cmd = "no {0}".format(want_route["dest"])
                        for item in key:
                            if "." in item or ":" in item or "/" in item:
                                cmd += " {0}".format(item)
                            else:
                                cmd += " vrf {0}".format(item)
                        update_commands.append(cmd)

                for key, value in iteritems(rotated_want_next_hops):
                    if key in rotated_have_next_hops:
                        existing = True
                        have_exit_point_attribs = rotated_have_next_hops[key]

                    else:
                        existing = False
                        have_exit_point_attribs = {}

                    updates = dict_diff(have_exit_point_attribs, value)

                    if updates or not existing:
                        update_commands.append(
                            self._compute_commands(
                                dest=want_route["dest"],
                                next_hop=key,
                                updates=updates,
                            )
                        )

            if update_commands:
                update_commands.insert(
                    0,
                    "address-family {0} {1}".format(
                        want_afi["afi"], want_afi["safi"]
                    ),
                )
                commands.extend(update_commands)

        if "vrf" in want and update_commands:
            commands.insert(0, "vrf {0}".format(want["vrf"]))

        return commands

    def _state_overridden(self, want, have):
        """ The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []

        # Iterate through all the entries, i.e., VRFs and Global entry in have
        # and fully remove the ones that are not present in want and then call
        # replaced

        for h_item in have:
            w_item = self._find_vrf(h_item, want)

            # Delete all the top-level keys (VRFs/Global Route Entry) that are
            # not specified in want.
            if not w_item:
                if "vrf" in h_item:
                    commands.append("no vrf {0}".format(h_item["vrf"]))
                else:
                    for have_afi in h_item.get("address_families", []):
                        commands.append(
                            "no address-family {0} {1}".format(
                                have_afi["afi"], have_afi["safi"]
                            )
                        )

            # For VRFs/Global Entry present in want, we also need to delete extraneous routes
            # from them. We cannot reuse `_state_replaced` for this purpose since its scope is
            # limited to replacing a single `dest`.
            else:
                del_cmds = []
                for have_afi in h_item.get("address_families", []):
                    want_afi = (
                        self.find_af_context(
                            have_afi, w_item.get("address_families", [])
                        )
                        or {}
                    )
                    update_commands = []
                    for h_route in have_afi.get("routes", []):
                        w_route = (
                            search_obj_in_list(
                                h_route["dest"],
                                want_afi.get("routes", []),
                                key="dest",
                            )
                            or {}
                        )
                        if not w_route:
                            update_commands.append(
                                "no {0}".format(h_route["dest"])
                            )

                    if update_commands:
                        update_commands.insert(
                            0,
                            "address-family {0} {1}".format(
                                want_afi["afi"], want_afi["safi"]
                            ),
                        )
                        del_cmds.extend(update_commands)

                if "vrf" in want and update_commands:
                    del_cmds.insert(0, "vrf {0}".format(want["vrf"]))

                commands.extend(del_cmds)

        # We finally call `_state_replaced` to replace exiting `dest` entries
        # or add new ones as specified in want.
        for w_item in want:
            h_item = self._find_vrf(w_item, have)
            commands.extend(
                self._state_replaced(remove_empties(w_item), h_item)
            )

        return commands

    def _state_merged(self, want, have):
        """ The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []

        for want_afi in want.get("address_families", []):
            have_afi = (
                self.find_af_context(
                    want_afi, have.get("address_families", [])
                )
                or {}
            )

            update_commands = []
            for want_route in want_afi.get("routes", []):
                have_route = (
                    search_obj_in_list(
                        want_route["dest"],
                        have_afi.get("routes", []),
                        key="dest",
                    )
                    or {}
                )

                # convert the next_hops list of dictionaries to dictionary of
                # dictionaries with (`dest_vrf`, `forward_router_address`, `interface`) tuple
                # being the key for each dictionary.
                # a combination of these 3 attributes uniquely identifies a route entry.
                # in case `dest_vrf` is not specified, `forward_router_address` and `interface`
                # become the unique identifier
                rotated_have_next_hops = self.rotate_next_hops(
                    have_route.get("next_hops", {})
                )
                rotated_want_next_hops = self.rotate_next_hops(
                    want_route.get("next_hops", {})
                )

                # for every dict in the want next_hops dictionaries, if the key
                # is present in `rotated_have_next_hops`, we set `existing` to True,
                # which means the the given want exit point exists and we run dict_diff
                # on `value` which is basically all the other attributes of the exit point
                # if the key is not present, it means that this is a new exit point
                for key, value in iteritems(rotated_want_next_hops):
                    if key in rotated_have_next_hops:
                        existing = True
                        have_exit_point_attribs = rotated_have_next_hops[key]

                    else:
                        existing = False
                        have_exit_point_attribs = {}

                    updates = dict_diff(have_exit_point_attribs, value)
                    if updates or not existing:
                        update_commands.append(
                            self._compute_commands(
                                dest=want_route["dest"],
                                next_hop=key,
                                # dict_merge() is necessary to make sure that we
                                # don't end up overridding the entry and also to
                                # allow incremental updates
                                updates=dict_merge(
                                    rotated_have_next_hops.get(key, {}),
                                    updates,
                                ),
                            )
                        )

            if update_commands:
                update_commands.insert(
                    0,
                    "address-family {0} {1}".format(
                        want_afi["afi"], want_afi["safi"]
                    ),
                )
                commands.extend(update_commands)

        if "vrf" in want and update_commands:
            commands.insert(0, "vrf {0}".format(want["vrf"]))

        return commands

    def _state_deleted(self, want, have):
        """ The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        if "address_families" not in want:
            return ["no vrf {0}".format(want["vrf"])]

        else:
            for want_afi in want.get("address_families", []):
                have_afi = (
                    self.find_af_context(
                        want_afi, have.get("address_families", [])
                    )
                    or {}
                )
                if have_afi:
                    commands.append(
                        "no address-family {0} {1}".format(
                            have_afi["afi"], have_afi["safi"]
                        )
                    )
            if "vrf" in want and commands:
                commands.insert(0, "vrf {0}".format(want["vrf"]))

        return commands

    def _find_vrf(self, item, entries):
        """ This method iterates through the items
            in `entries` and returns the object that
            matches `item`.

        :rtype: A dict
        :returns: the obj in `entries` that matches `item`
        """
        obj = {}
        afi = item.get("vrf")

        if afi:
            obj = search_obj_in_list(afi, entries, key="vrf") or {}
        else:
            for x in entries:
                if "vrf" not in remove_empties(x):
                    obj = x
                    break
        return obj

    def find_af_context(self, want_af_context, have_address_families):
        """ This method iterates through the have AFs
            and returns the one that matches the want AF

        :rtype: A dict
        :returns: the corresponding AF in have AFs
                  that matches the want AF
        """
        for have_af in have_address_families:
            if (
                have_af["afi"] == want_af_context["afi"]
                and have_af["safi"] == want_af_context["safi"]
            ):
                return have_af

    def rotate_next_hops(self, next_hops):
        """ This method iterates through the list of
            next hops for a given destination network
            and converts it to a dictionary of dictionaries.
            Each dictionary has a primary key indicated by the
            tuple of `dest_vrf`, `forward_router_address` and
            `interface` and the value of this key is a dictionary
            that contains all the other attributes of the next hop.

        :rtype: A dict
        :returns: A next_hops list in a dictionary of dictionaries format
        """
        next_hops_dict = {}

        for entry in next_hops:
            entry = entry.copy()
            key_list = []

            for x in ["dest_vrf", "forward_router_address", "interface"]:
                if entry.get(x):
                    key_list.append(entry.pop(x))

            key = tuple(key_list)
            next_hops_dict[key] = entry

        return next_hops_dict

    def _compute_commands(self, dest, next_hop, updates=None):
        """ This method computes a static route entry command
            from the specified `dest`, `next_hop` and `updates`

        :rtype: A str
        :returns: A platform specific static routes command
        """
        if not updates:
            updates = {}

        command = dest

        for x in next_hop:
            if "." in x or ":" in x or "/" in x:
                command += " {0}".format(x)
            else:
                command += " vrf {0}".format(x)

        for key in sorted(updates):
            if key == "admin_distance":
                command += " {0}".format(updates[key])
            else:
                command += " {0} {1}".format(key, updates[key])

        return command



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2016, Jonathan Mainguy <jon@soh.re>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# basis of code taken from the ansible twillio and nexmo modules

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: catapult
short_description: Send a sms / mms using the catapult bandwidth api
description:
     - Allows notifications to be sent using sms / mms via the catapult bandwidth api.
options:
  src:
    description:
      - One of your catapult telephone numbers the message should come from (must be in E.164 format, like C(+19195551212)).
    required: true
  dest:
    description:
      - The phone number or numbers the message should be sent to (must be in E.164 format, like C(+19195551212)).
    required: true
  msg:
    description:
      - The contents of the text message (must be 2048 characters or less).
    required: true
  media:
    description:
      - For MMS messages, a media url to the location of the media to be sent with the message.
  user_id:
    description:
      - User Id from Api account page.
    required: true
  api_token:
    description:
      - Api Token from Api account page.
    required: true
  api_secret:
    description:
      - Api Secret from Api account page.
    required: true

author: "Jonathan Mainguy (@Jmainguy)"
notes:
    - Will return changed even if the media url is wrong.
    - Will return changed if the destination number is invalid.

'''

EXAMPLES = '''
- name: Send a mms to multiple users
  community.general.catapult:
    src: "+15035555555"
    dest:
      - "+12525089000"
      - "+12018994225"
    media: "http://example.com/foobar.jpg"
    msg: "Task is complete"
    user_id: "{{ user_id }}"
    api_token: "{{ api_token }}"
    api_secret: "{{ api_secret }}"

- name: Send a sms to a single user
  community.general.catapult:
    src: "+15035555555"
    dest: "+12018994225"
    msg: "Consider yourself notified"
    user_id: "{{ user_id }}"
    api_token: "{{ api_token }}"
    api_secret: "{{ api_secret }}"

'''

RETURN = '''
changed:
    description: Whether the api accepted the message.
    returned: always
    type: bool
    sample: True
'''


import json

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def send(module, src, dest, msg, media, user_id, api_token, api_secret):
    """
    Send the message
    """
    AGENT = "Ansible"
    URI = "https://api.catapult.inetwork.com/v1/users/%s/messages" % user_id
    data = {'from': src, 'to': dest, 'text': msg}
    if media:
        data['media'] = media

    headers = {'User-Agent': AGENT, 'Content-type': 'application/json'}

    # Hack module params to have the Basic auth params that fetch_url expects
    module.params['url_username'] = api_token.replace('\n', '')
    module.params['url_password'] = api_secret.replace('\n', '')

    return fetch_url(module, URI, data=json.dumps(data), headers=headers, method="post")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            src=dict(required=True),
            dest=dict(required=True, type='list'),
            msg=dict(required=True),
            user_id=dict(required=True),
            api_token=dict(required=True, no_log=True),
            api_secret=dict(required=True, no_log=True),
            media=dict(default=None, required=False),
        ),
    )

    src = module.params['src']
    dest = module.params['dest']
    msg = module.params['msg']
    media = module.params['media']
    user_id = module.params['user_id']
    api_token = module.params['api_token']
    api_secret = module.params['api_secret']

    for number in dest:
        rc, info = send(module, src, number, msg, media, user_id, api_token, api_secret)
        if info["status"] != 201:
            body = json.loads(info["body"])
            fail_msg = body["message"]
            module.fail_json(msg=fail_msg)

    changed = True
    module.exit_json(changed=changed)


if __name__ == '__main__':
    main()



# Copyright: (c) 2019, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.icx import icx_linkagg
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .icx_module import TestICXModule, load_fixture


class TestICXLinkaggModule(TestICXModule):

    module = icx_linkagg

    def setUp(self):
        super(TestICXLinkaggModule, self).setUp()
        self.mock_get_config = patch('ansible_collections.community.network.plugins.modules.network.icx.icx_linkagg.get_config')
        self.get_config = self.mock_get_config.start()
        self.mock_load_config = patch('ansible_collections.community.network.plugins.modules.network.icx.icx_linkagg.load_config')
        self.load_config = self.mock_load_config.start()
        self.mock_exec_command = patch('ansible_collections.community.network.plugins.modules.network.icx.icx_linkagg.exec_command')
        self.exec_command = self.mock_exec_command.start()
        self.set_running_config()

    def tearDown(self):
        super(TestICXLinkaggModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_exec_command.stop()

    def load_fixtures(self, commands=None):
        compares = None

        def load_from_file(*args, **kwargs):
            module = args
            for arg in args:
                if arg.params['check_running_config'] is True:
                    return load_fixture('lag_running_config.txt').strip()
                else:
                    return ''

        self.get_config.side_effect = load_from_file
        self.load_config.return_value = None

    def test_icx_linkage_create_new_LAG(self):
        set_module_args(dict(group=10, name="LAG3", mode='static', members=['ethernet 1/1/4 to ethernet 1/1/7']))
        if not self.ENV_ICX_USE_DIFF:
            commands = ['lag LAG3 static id 10', 'ports ethernet 1/1/4 to ethernet 1/1/7', 'exit']
            self.execute_module(commands=commands, changed=True)
        else:
            commands = ['lag LAG3 static id 10', 'ports ethernet 1/1/4 to ethernet 1/1/7', 'exit']
            self.execute_module(commands=commands, changed=True)

    def test_icx_linkage_modify_LAG(self):
        set_module_args(dict(group=100, name="LAG1", mode='dynamic', members=['ethernet 1/1/4 to 1/1/7']))
        if not self.ENV_ICX_USE_DIFF:
            commands = [
                'lag LAG1 dynamic id 100',
                'ports ethernet 1/1/4 to 1/1/7',
                'exit'
            ]
            self.execute_module(commands=commands, changed=True)
        else:
            commands = [
                'lag LAG1 dynamic id 100',
                'no ports ethernet 1/1/3',
                'no ports ethernet 1/1/8',
                'ports ethernet 1/1/4',
                'exit'
            ]
            self.execute_module(commands=commands, changed=True)

    def test_icx_linkage_modify_LAG_compare(self):
        set_module_args(dict(group=100, name="LAG1", mode='dynamic', members=['ethernet 1/1/4 to 1/1/7'], check_running_config=True))
        if self.get_running_config(compare=True):
            if not self.ENV_ICX_USE_DIFF:
                commands = [
                    'lag LAG1 dynamic id 100',
                    'no ports ethernet 1/1/3',
                    'no ports ethernet 1/1/8',
                    'ports ethernet 1/1/4',
                    'exit'
                ]
                self.execute_module(commands=commands, changed=True)
            else:
                commands = [
                    'lag LAG1 dynamic id 100',
                    'no ports ethernet 1/1/3',
                    'no ports ethernet 1/1/8',
                    'ports ethernet 1/1/4',
                    'exit'
                ]
                self.execute_module(commands=commands, changed=True)

    def test_icx_linkage_purge_LAG(self):
        set_module_args(dict(aggregate=[dict(group=100, name="LAG1", mode='dynamic')], purge=True))
        if not self.ENV_ICX_USE_DIFF:
            commands = [
                'lag LAG1 dynamic id 100',
                'exit'
            ]
            self.execute_module(commands=commands, changed=True)
        else:
            commands = [
                'lag LAG1 dynamic id 100',
                'exit',
                'no lag LAG2 dynamic id 200'
            ]
            self.execute_module(commands=commands, changed=True)

    def test_icx_linkage_remove_LAG(self):
        set_module_args(dict(group=100, name="LAG1", mode='dynamic', members=['ethernet 1/1/4 to 1/1/7'], state='absent'))
        if not self.ENV_ICX_USE_DIFF:
            commands = [
                'no lag LAG1 dynamic id 100'
            ]
            self.execute_module(commands=commands, changed=True)
        else:
            commands = [
                'no lag LAG1 dynamic id 100'
            ]
            self.execute_module(commands=commands, changed=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Hiroyuki Matsuo <h.matsuo.engineer@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: rabbitmq_vhost_limits
author: Hiroyuki Matsuo (@h-matsuo)

short_description: Manage the state of virtual host limits in RabbitMQ
description:
    - This module sets/clears certain limits on a virtual host.
    - The configurable limits are I(max_connections) and I(max-queues).

options:
    max_connections:
        description:
            - Max number of concurrent client connections.
            - Negative value means "no limit".
            - Ignored when the I(state) is C(absent).
        default: -1
    max_queues:
        description:
            - Max number of queues.
            - Negative value means "no limit".
            - Ignored when the I(state) is C(absent).
        default: -1
    node:
        description:
            - Name of the RabbitMQ Erlang node to manage.
    state:
        description:
            - Specify whether the limits are to be set or cleared.
            - If set to C(absent), the limits of both I(max_connections) and I(max-queues) will be cleared.
        default: present
        choices: [present, absent]
    vhost:
        description:
            - Name of the virtual host to manage.
        default: /
'''

EXAMPLES = '''
# Limit both of the max number of connections and queues on the vhost '/'.
- rabbitmq_vhost_limits:
    vhost: /
    max_connections: 64
    max_queues: 256
    state: present

# Limit the max number of connections on the vhost '/'.
# This task implicitly clears the max number of queues limit using default value: -1.
- rabbitmq_vhost_limits:
    vhost: /
    max_connections: 64
    state: present

# Clear the limits on the vhost '/'.
- rabbitmq_vhost_limits:
    vhost: /
    state: absent
'''

RETURN = ''' # '''


import json
from ansible.module_utils.basic import AnsibleModule


class RabbitMqVhostLimits(object):
    def __init__(self, module):
        self._module = module
        self._max_connections = module.params['max_connections']
        self._max_queues = module.params['max_queues']
        self._node = module.params['node']
        self._state = module.params['state']
        self._vhost = module.params['vhost']
        self._rabbitmqctl = module.get_bin_path('rabbitmqctl', True)

    def _exec(self, args):
        cmd = [self._rabbitmqctl, '-q', '-p', self._vhost]
        if self._node is not None:
            cmd.extend(['-n', self._node])
        rc, out, err = self._module.run_command(cmd + args, check_rc=True)
        return dict(rc=rc, out=out.splitlines(), err=err.splitlines())

    def list(self):
        exec_result = self._exec(['list_vhost_limits'])
        vhost_limits = exec_result['out'][0]
        max_connections = None
        max_queues = None
        if vhost_limits:
            vhost_limits = json.loads(vhost_limits)
            if 'max-connections' in vhost_limits:
                max_connections = vhost_limits['max-connections']
            if 'max-queues' in vhost_limits:
                max_queues = vhost_limits['max-queues']
        return dict(
            max_connections=max_connections,
            max_queues=max_queues
        )

    def set(self):
        if self._module.check_mode:
            return
        json_str = '{{"max-connections": {0}, "max-queues": {1}}}'.format(self._max_connections, self._max_queues)
        self._exec(['set_vhost_limits', json_str])

    def clear(self):
        if self._module.check_mode:
            return
        self._exec(['clear_vhost_limits'])


def main():
    arg_spec = dict(
        max_connections=dict(default=-1, type='int'),
        max_queues=dict(default=-1, type='int'),
        node=dict(default=None, type='str'),
        state=dict(default='present', choices=['present', 'absent'], type='str'),
        vhost=dict(default='/', type='str')
    )

    module = AnsibleModule(
        argument_spec=arg_spec,
        supports_check_mode=True
    )

    max_connections = module.params['max_connections']
    max_queues = module.params['max_queues']
    node = module.params['node']
    state = module.params['state']
    vhost = module.params['vhost']

    module_result = dict(changed=False)
    rabbitmq_vhost_limits = RabbitMqVhostLimits(module)
    current_status = rabbitmq_vhost_limits.list()

    if state == 'present':
        wanted_status = dict(
            max_connections=max_connections,
            max_queues=max_queues
        )
    else:  # state == 'absent'
        wanted_status = dict(
            max_connections=None,
            max_queues=None
        )

    if current_status != wanted_status:
        module_result['changed'] = True
        if state == 'present':
            rabbitmq_vhost_limits.set()
        else:  # state == 'absent'
            rabbitmq_vhost_limits.clear()

    module.exit_json(**module_result)


if __name__ == '__main__':
    main()



# (c) 2019 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.cloudengine import ce_static_route_bfd
from ansible_collections.community.network.tests.unit.plugins.modules.network.cloudengine.ce_module import TestCloudEngineModule, load_fixture
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args


class TestCloudEngineLacpModule(TestCloudEngineModule):
    module = ce_static_route_bfd

    def setUp(self):
        super(TestCloudEngineLacpModule, self).setUp()

        self.mock_get_config = patch('ansible_collections.community.network.plugins.modules.network.cloudengine.ce_static_route_bfd.get_nc_config')
        self.get_nc_config = self.mock_get_config.start()

        self.mock_set_config = patch('ansible_collections.community.network.plugins.modules.network.cloudengine.ce_static_route_bfd.set_nc_config')
        self.set_nc_config = self.mock_set_config.start()
        self.set_nc_config.return_value = load_fixture('ce_lldp', 'result_ok.txt')

    def tearDown(self):
        super(TestCloudEngineLacpModule, self).tearDown()
        self.mock_set_config.stop()
        self.mock_get_config.stop()

    def test_ce_static_route_bfd_changed_false(self):
        srBfdPara_1 = load_fixture('ce_static_route_bfd', 'srBfdPara_1.txt')
        staticrtbase_1 = load_fixture('ce_static_route_bfd', 'staticrtbase_1.txt')
        self.get_nc_config.side_effect = (srBfdPara_1, srBfdPara_1, staticrtbase_1, staticrtbase_1)

        config = dict(
            prefix='255.255.0.0',
            mask=22,
            aftype='v4',
            next_hop='10.10.1.1',
            nhp_interface='10GE1/0/1',
            vrf='mgnt',
            destvrf='_public_',
            tag=23,
            description='for a test',
            pref='22',
            function_flag='dynamicBFD',
            min_tx_interval='32',
            min_rx_interval='23',
            detect_multiplier='24',
            bfd_session_name='43'
        )
        set_module_args(config)
        self.execute_module(changed=False)

    def test_ce_static_route_bfd_changed_true(self):
        srBfdPara_1 = load_fixture('ce_static_route_bfd', 'srBfdPara_1.txt')
        srBfdPara_2 = load_fixture('ce_static_route_bfd', 'srBfdPara_2.txt')
        staticrtbase_1 = load_fixture('ce_static_route_bfd', 'staticrtbase_1.txt')
        staticrtbase_2 = load_fixture('ce_static_route_bfd', 'staticrtbase_2.txt')
        self.get_nc_config.side_effect = (srBfdPara_1, staticrtbase_1, srBfdPara_2, staticrtbase_2)
        updates = ['ip route-static vpn-instance mgnt 255.255.0.0 255.255.252.0 10GE1/0/1 10.10.1.1',
                   ' preference 22',
                   ' tag 23',
                   ' track bfd-session 43',
                   ' description for a test']
        config = dict(
            prefix='255.255.0.0',
            mask=22,
            aftype='v4',
            next_hop='10.10.1.1',
            nhp_interface='10GE1/0/1',
            vrf='mgnt',
            destvrf='_public_',
            tag=23,
            description='for a test',
            pref='22',
            function_flag='dynamicBFD',
            min_tx_interval='32',
            min_rx_interval='23',
            detect_multiplier='24',
            bfd_session_name='43'
        )
        set_module_args(config)
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result['updates']), sorted(updates))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: ipa_hostgroup
author: Thomas Krahn (@Nosmoht)
short_description: Manage FreeIPA host-group
description:
- Add, modify and delete an IPA host-group using IPA API.
options:
  cn:
    description:
    - Name of host-group.
    - Can not be changed as it is the unique identifier.
    required: true
    aliases: ["name"]
    type: str
  description:
    description:
    - Description.
    type: str
  host:
    description:
    - List of hosts that belong to the host-group.
    - If an empty list is passed all hosts will be removed from the group.
    - If option is omitted hosts will not be checked or changed.
    - If option is passed all assigned hosts that are not passed will be unassigned from the group.
    type: list
    elements: str
  hostgroup:
    description:
    - List of host-groups than belong to that host-group.
    - If an empty list is passed all host-groups will be removed from the group.
    - If option is omitted host-groups will not be checked or changed.
    - If option is passed all assigned hostgroups that are not passed will be unassigned from the group.
    type: list
    elements: str
  state:
    description:
    - State to ensure.
    default: "present"
    choices: ["absent", "disabled", "enabled", "present"]
    type: str
extends_documentation_fragment:
- community.general.ipa.documentation

'''

EXAMPLES = r'''
- name: Ensure host-group databases is present
  community.general.ipa_hostgroup:
    name: databases
    state: present
    host:
    - db.example.com
    hostgroup:
    - mysql-server
    - oracle-server
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure host-group databases is absent
  community.general.ipa_hostgroup:
    name: databases
    state: absent
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret
'''

RETURN = r'''
hostgroup:
  description: Hostgroup as returned by IPA API.
  returned: always
  type: dict
'''

import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.ipa import IPAClient, ipa_argument_spec
from ansible.module_utils._text import to_native


class HostGroupIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super(HostGroupIPAClient, self).__init__(module, host, port, protocol)

    def hostgroup_find(self, name):
        return self._post_json(method='hostgroup_find', name=None, item={'all': True, 'cn': name})

    def hostgroup_add(self, name, item):
        return self._post_json(method='hostgroup_add', name=name, item=item)

    def hostgroup_mod(self, name, item):
        return self._post_json(method='hostgroup_mod', name=name, item=item)

    def hostgroup_del(self, name):
        return self._post_json(method='hostgroup_del', name=name)

    def hostgroup_add_member(self, name, item):
        return self._post_json(method='hostgroup_add_member', name=name, item=item)

    def hostgroup_add_host(self, name, item):
        return self.hostgroup_add_member(name=name, item={'host': item})

    def hostgroup_add_hostgroup(self, name, item):
        return self.hostgroup_add_member(name=name, item={'hostgroup': item})

    def hostgroup_remove_member(self, name, item):
        return self._post_json(method='hostgroup_remove_member', name=name, item=item)

    def hostgroup_remove_host(self, name, item):
        return self.hostgroup_remove_member(name=name, item={'host': item})

    def hostgroup_remove_hostgroup(self, name, item):
        return self.hostgroup_remove_member(name=name, item={'hostgroup': item})


def get_hostgroup_dict(description=None):
    data = {}
    if description is not None:
        data['description'] = description
    return data


def get_hostgroup_diff(client, ipa_hostgroup, module_hostgroup):
    return client.get_diff(ipa_data=ipa_hostgroup, module_data=module_hostgroup)


def ensure(module, client):
    name = module.params['cn']
    state = module.params['state']
    host = module.params['host']
    hostgroup = module.params['hostgroup']

    ipa_hostgroup = client.hostgroup_find(name=name)
    module_hostgroup = get_hostgroup_dict(description=module.params['description'])

    changed = False
    if state == 'present':
        if not ipa_hostgroup:
            changed = True
            if not module.check_mode:
                ipa_hostgroup = client.hostgroup_add(name=name, item=module_hostgroup)
        else:
            diff = get_hostgroup_diff(client, ipa_hostgroup, module_hostgroup)
            if len(diff) > 0:
                changed = True
                if not module.check_mode:
                    data = {}
                    for key in diff:
                        data[key] = module_hostgroup.get(key)
                    client.hostgroup_mod(name=name, item=data)

        if host is not None:
            changed = client.modify_if_diff(name, ipa_hostgroup.get('member_host', []), [item.lower() for item in host],
                                            client.hostgroup_add_host, client.hostgroup_remove_host) or changed

        if hostgroup is not None:
            changed = client.modify_if_diff(name, ipa_hostgroup.get('member_hostgroup', []),
                                            [item.lower() for item in hostgroup],
                                            client.hostgroup_add_hostgroup,
                                            client.hostgroup_remove_hostgroup) or changed

    else:
        if ipa_hostgroup:
            changed = True
            if not module.check_mode:
                client.hostgroup_del(name=name)

    return changed, client.hostgroup_find(name=name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(cn=dict(type='str', required=True, aliases=['name']),
                         description=dict(type='str'),
                         host=dict(type='list', elements='str'),
                         hostgroup=dict(type='list', elements='str'),
                         state=dict(type='str', default='present', choices=['present', 'absent', 'enabled', 'disabled']))

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    client = HostGroupIPAClient(module=module,
                                host=module.params['ipa_host'],
                                port=module.params['ipa_port'],
                                protocol=module.params['ipa_prot'])

    try:
        client.login(username=module.params['ipa_user'],
                     password=module.params['ipa_pass'])
        changed, hostgroup = ensure(module, client)
        module.exit_json(changed=changed, hostgroup=hostgroup)
    except Exception as e:
        module.fail_json(msg=to_native(e), exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Yunge Zhu, <yungez@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_applicationsecuritygroup_info
short_description: Get Azure Application Security Group facts
description:
    - Get facts of Azure Application Security Group.

options:
    resource_group:
        description:
            - The name of the resource group.
    name:
        description:
            - The name of the application security group.
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.

extends_documentation_fragment:
- azure.azcollection.azure


author:
    - Yunge Zhu (@yungezz)

'''

EXAMPLES = '''
  - name: List application security groups in specific resource group
    azure_rm_applicationsecuritygroup_info:
      resource_group: myResourceGroup

  - name: List application security groups in specific subscription
    azure_rm_applicationsecuritygroup_info:

  - name: Get application security group by name
    azure_rm_applicationsecuritygroup_info:
        resource_group: myResourceGroup
        name: myApplicationSecurityGroup
        tags:
            - foo
'''

RETURN = '''
applicationsecuritygroups:
    description:
        - List of application security groups.
    returned: always
    type: complex
    contains:
        id:
            description: Id of the application security group.
            type: str
            returned: always
            sample:
                "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/applicationSecurityGroups/MyAsg"
        location:
            description:
                - Location of the application security group.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - Name of the resource.
            type: str
            returned: always
            sample: myAsg
        provisioning_state:
            description:
                - Provisioning state of application security group.
            type: str
            returned: always
            sample: Succeeded
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from msrestazure.azure_exceptions import CloudError
    from msrest.polling import LROPoller
    from msrestazure.azure_operation import AzureOperationPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


def applicationsecuritygroup_to_dict(asg):
    return dict(
        id=asg.id,
        location=asg.location,
        name=asg.name,
        tags=asg.tags,
        provisioning_state=asg.provisioning_state
    )


class AzureRMApplicationSecurityGroupInfo(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str'
            ),
            name=dict(
                type='str'
            ),
            tags=dict(type='list')
        )

        self.resource_group = None
        self.name = None
        self.tags = None

        self.results = dict(changed=False)

        super(AzureRMApplicationSecurityGroupInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                                  supports_check_mode=False,
                                                                  supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        is_old_facts = self.module._name == 'azure_rm_applicationsecuritygroup_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_applicationsecuritygroup_facts' module has been renamed to 'azure_rm_applicationsecuritygroup_info'",
                                  version='2.13')

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        if self.name:
            if self.resource_group:
                self.results['applicationsecuritygroups'] = self.get()
            else:
                self.fail("resource_group is required when filtering by name")
        elif self.resource_group:
            self.results['applicationsecuritygroups'] = self.list_by_resource_group()
        else:
            self.results['applicationsecuritygroups'] = self.list_all()

        return self.results

    def get(self):
        '''
        Gets the properties of the specified Application Security Group.

        :return: deserialized Application Security Group instance state dictionary
        '''
        self.log("Get the Application Security Group instance {0}".format(self.name))

        results = []
        try:
            response = self.network_client.application_security_groups.get(resource_group_name=self.resource_group,
                                                                           application_security_group_name=self.name)
            self.log("Response : {0}".format(response))

            if response and self.has_tags(response.tags, self.tags):
                results.append(applicationsecuritygroup_to_dict(response))
        except CloudError as e:
            self.fail('Did not find the Application Security Group instance.')
        return results

    def list_by_resource_group(self):
        '''
        Lists the properties of Application Security Groups in specific resource group.

        :return: deserialized Application Security Group instance state dictionary
        '''
        self.log("Get the Application Security Groups in resource group {0}".format(self.resource_group))

        results = []
        try:
            response = list(self.network_client.application_security_groups.list(resource_group_name=self.resource_group))
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    if self.has_tags(item.tags, self.tags):
                        results.append(applicationsecuritygroup_to_dict(item))
        except CloudError as e:
            self.log('Did not find the Application Security Group instance.')
        return results

    def list_all(self):
        '''
        Lists the properties of Application Security Groups in specific subscription.

        :return: deserialized Application Security Group instance state dictionary
        '''
        self.log("Get the Application Security Groups in current subscription")

        results = []
        try:
            response = list(self.network_client.application_security_groups.list_all())
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    if self.has_tags(item.tags, self.tags):
                        results.append(applicationsecuritygroup_to_dict(item))
        except CloudError as e:
            self.log('Did not find the Application Security Group instance.')
        return results


def main():
    """Main execution"""
    AzureRMApplicationSecurityGroupInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2015, Hans-Joachim Kliemeck <git@kliemeck.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_owner
short_description: Set owner
description:
    - Set owner of files or directories.
options:
  path:
    description:
      - Path to be used for changing owner.
    type: path
    required: yes
  user:
    description:
      - Name to be used for changing owner.
    type: str
    required: yes
  recurse:
    description:
      - Indicates if the owner should be changed recursively.
    type: bool
    default: no
seealso:
- module: ansible.windows.win_acl
- module: ansible.windows.win_file
- module: ansible.windows.win_stat
author:
- Hans-Joachim Kliemeck (@h0nIg)
'''

EXAMPLES = r'''
- name: Change owner of path
  ansible.windows.win_owner:
    path: C:\apache
    user: apache
    recurse: yes

- name: Set the owner of root directory
  ansible.windows.win_owner:
    path: C:\apache
    user: SYSTEM
    recurse: no
'''

RETURN = r'''

'''



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: digital_ocean_domain
short_description: Create/delete a DNS domain in DigitalOcean
description:
     - Create/delete a DNS domain in DigitalOcean.
author: "Michael Gregson (@mgregson)"
options:
  state:
    description:
     - Indicate desired state of the target.
    default: present
    choices: ['present', 'absent']
  id:
    description:
     - Numeric, the droplet id you want to operate on.
    aliases: ['droplet_id']
  name:
    description:
     - String, this is the name of the droplet - must be formatted by hostname rules, or the name of a SSH key, or the name of a domain.
  ip:
    description:
     - An 'A' record for '@' ($ORIGIN) will be created with the value 'ip'.  'ip' is an IP version 4 address.
extends_documentation_fragment:
- community.digitalocean.digital_ocean.documentation

notes:
  - Environment variables DO_OAUTH_TOKEN can be used for the oauth_token.
  - As of Ansible 1.9.5 and 2.0, Version 2 of the DigitalOcean API is used, this removes C(client_id) and C(api_key) options in favor of C(oauth_token).
  - If you are running Ansible 1.9.4 or earlier you might not be able to use the included version of this module as the API version used has been retired.

requirements:
  - "python >= 2.6"
'''


EXAMPLES = '''
- name: Create a domain
  digital_ocean_domain:
    state: present
    name: my.digitalocean.domain
    ip: 127.0.0.1

# Create a droplet and corresponding domain
- name: Create a droplet
  digital_ocean:
    state: present
    name: test_droplet
    size_id: 1gb
    region_id: sgp1
    image_id: ubuntu-14-04-x64
  register: test_droplet

- name: Create a corresponding domain
  digital_ocean_domain:
    state: present
    name: "{{ test_droplet.droplet.name }}.my.domain"
    ip: "{{ test_droplet.droplet.ip_address }}"

'''

import traceback
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import DigitalOceanHelper
from ansible.module_utils._text import to_native


class DoManager(DigitalOceanHelper, object):
    def __init__(self, module):
        super(DoManager, self).__init__(module)
        self.domain_name = module.params.get('name', None)
        self.domain_ip = module.params.get('ip', None)
        self.domain_id = module.params.get('id', None)

    @staticmethod
    def jsonify(response):
        return response.status_code, response.json

    def all_domains(self):
        resp = self.get('domains/')
        return resp

    def find(self):
        if self.domain_name is None and self.domain_id is None:
            return False

        domains = self.all_domains()
        status, json = self.jsonify(domains)
        for domain in json['domains']:
            if domain['name'] == self.domain_name:
                return True
        return False

    def add(self):
        params = {'name': self.domain_name, 'ip_address': self.domain_ip}
        resp = self.post('domains/', data=params)
        status = resp.status_code
        json = resp.json
        if status == 201:
            return json['domain']
        else:
            return json

    def all_domain_records(self):
        resp = self.get('domains/%s/records/' % self.domain_name)
        return resp.json

    def domain_record(self):
        resp = self.get('domains/%s' % self.domain_name)
        status, json = self.jsonify(resp)
        return json

    def destroy_domain(self):
        resp = self.delete('domains/%s' % self.domain_name)
        status, json = self.jsonify(resp)
        if status == 204:
            return True
        else:
            return json

    def edit_domain_record(self, record):
        params = {'name': '@',
                  'data': self.module.params.get('ip')}
        resp = self.put('domains/%s/records/%s' % (self.domain_name, record['id']), data=params)
        status, json = self.jsonify(resp)

        return json['domain_record']

    def create_domain_record(self):
        params = {'name': '@',
                  'type': 'A',
                  'data': self.module.params.get('ip')}

        resp = self.post('domains/%s/records' % (self.domain_name), data=params)
        status, json = self.jsonify(resp)

        return json['domain_record']


def core(module):
    do_manager = DoManager(module)
    state = module.params.get('state')

    domain = do_manager.find()
    if state == 'present':
        if not domain:
            domain = do_manager.add()
            if 'message' in domain:
                module.fail_json(changed=False, msg=domain['message'])
            else:
                module.exit_json(changed=True, domain=domain)
        else:
            records = do_manager.all_domain_records()
            at_record = None
            for record in records['domain_records']:
                if record['name'] == "@" and record['type'] == 'A':
                    at_record = record

            if not at_record:
                do_manager.create_domain_record()
                module.exit_json(changed=True, domain=do_manager.find())
            elif not at_record['data'] == module.params.get('ip'):
                do_manager.edit_domain_record(at_record)
                module.exit_json(changed=True, domain=do_manager.find())
            else:
                module.exit_json(changed=False, domain=do_manager.domain_record())

    elif state == 'absent':
        if not domain:
            module.exit_json(changed=False, msg="Domain not found")
        else:
            delete_event = do_manager.destroy_domain()
            if not delete_event:
                module.fail_json(changed=False, msg=delete_event['message'])
            else:
                module.exit_json(changed=True, event=None)
        delete_event = do_manager.destroy_domain()
        module.exit_json(changed=delete_event)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        state=dict(choices=['present', 'absent'], default='present'),
        name=dict(type='str'),
        id=dict(aliases=['droplet_id'], type='int'),
        ip=dict(type='str')
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=(
            ['id', 'name'],
        ),
    )

    try:
        core(module)
    except Exception as e:
        module.fail_json(msg=to_native(e), exception=traceback.format_exc())


if __name__ == '__main__':
    main()



# Copyright (c) 2018, Oracle and/or its affiliates.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):
    DOCUMENTATION = """
    options:
        force_create:
            description: Whether to attempt non-idempotent creation of a resource. By default, create resource is an
                         idempotent operation, and doesn't create the resource if it already exists. Setting this option
                         to true, forcefully creates a copy of the resource, even if it already exists.This option is
                         mutually exclusive with I(key_by).
            default: False
            type: bool
        key_by:
            description: The list of comma-separated attributes of this resource which should be used to uniquely
                         identify an instance of the resource. By default, all the attributes of a resource except
                         I(freeform_tags) are used to uniquely identify a resource.
            type: list
    """



#!/usr/bin/python
#
# Copyright (c) 2018 Yunge Zhu, <yungez@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_webapp_info


short_description: Get Azure web app facts

description:
    - Get facts for a specific web app or all web app in a resource group, or all web app in current subscription.

options:
    name:
        description:
            - Only show results for a specific web app.
    resource_group:
        description:
            - Limit results by resource group.
    return_publish_profile:
        description:
            - Indicate whether to return publishing profile of the web app.
        default: False
        type: bool
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.

extends_documentation_fragment:
- azure.azcollection.azure


author:
    - Yunge Zhu (@yungezz)
'''

EXAMPLES = '''
    - name: Get facts for web app by name
      azure_rm_webapp_info:
        resource_group: myResourceGroup
        name: winwebapp1

    - name: Get facts for web apps in resource group
      azure_rm_webapp_info:
        resource_group: myResourceGroup

    - name: Get facts for web apps with tags
      azure_rm_webapp_info:
        tags:
          - testtag
          - foo:bar
'''

RETURN = '''
webapps:
    description:
        - List of web apps.
    returned: always
    type: complex
    contains:
        id:
            description:
                - ID of the web app.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Web/sites/myWebApp
        name:
            description:
                - Name of the web app.
            returned: always
            type: str
            sample: winwebapp1
        resource_group:
            description:
                - Resource group of the web app.
            returned: always
            type: str
            sample: myResourceGroup
        location:
            description:
                - Location of the web app.
            returned: always
            type: str
            sample: eastus
        plan:
            description:
                - ID of app service plan used by the web app.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Web/serverfarms/myAppServicePlan
        app_settings:
            description:
                - App settings of the application. Only returned when web app has app settings.
            returned: always
            type: dict
            sample: {
                    "testkey": "testvalue",
                    "testkey2": "testvalue2"
                    }
        frameworks:
            description:
                - Frameworks of the application. Only returned when web app has frameworks.
            returned: always
            type: list
            sample: [
                    {
                        "name": "net_framework",
                        "version": "v4.0"
                    },
                    {
                        "name": "java",
                        "settings": {
                            "java_container": "tomcat",
                            "java_container_version": "8.5"
                        },
                        "version": "1.7"
                    },
                    {
                        "name": "php",
                        "version": "5.6"
                    }
                    ]
        availability_state:
            description:
                - Availability of this web app.
            returned: always
            type: str
            sample: Normal
        default_host_name:
            description:
                - Host name of the web app.
            returned: always
            type: str
            sample: vxxisurg397winapp4.azurewebsites.net
        enabled:
            description:
                - Indicates the web app enabled or not.
            returned: always
            type: bool
            sample: true
        enabled_host_names:
            description:
                - Enabled host names of the web app.
            returned: always
            type: list
            sample: [
                    "vxxisurg397winapp4.azurewebsites.net",
                    "vxxisurg397winapp4.scm.azurewebsites.net"
                    ]
        host_name_ssl_states:
            description:
                - SSL state per host names of the web app.
            returned: always
            type: list
            sample: [
                    {
                        "hostType": "Standard",
                        "name": "vxxisurg397winapp4.azurewebsites.net",
                        "sslState": "Disabled"
                    },
                    {
                        "hostType": "Repository",
                        "name": "vxxisurg397winapp4.scm.azurewebsites.net",
                        "sslState": "Disabled"
                    }
                    ]
        host_names:
            description:
                - Host names of the web app.
            returned: always
            type: list
            sample: [
                    "vxxisurg397winapp4.azurewebsites.net"
                    ]
        outbound_ip_addresses:
            description:
                - Outbound IP address of the web app.
            returned: always
            type: str
            sample: "40.71.11.131,40.85.166.200,168.62.166.67,137.135.126.248,137.135.121.45"
        ftp_publish_url:
            description:
                - Publishing URL of the web app when deployment type is FTP.
            returned: always
            type: str
            sample: ftp://xxxx.ftp.azurewebsites.windows.net
        state:
            description:
                - State of the web app.
            returned: always
            type: str
            sample: running
        publishing_username:
            description:
                - Publishing profile user name.
            returned: only when I(return_publish_profile=True).
            type: str
            sample: "$vxxisuRG397winapp4"
        publishing_password:
            description:
                - Publishing profile password.
            returned: only when I(return_publish_profile=True).
            type: str
            sample: "uvANsPQpGjWJmrFfm4Ssd5rpBSqGhjMk11pMSgW2vCsQtNx9tcgZ0xN26s9A"
        tags:
            description:
               - Tags assigned to the resource. Dictionary of string:string pairs.
            returned: always
            type: dict
            sample: { tag1: abc }
'''
try:
    from msrestazure.azure_exceptions import CloudError
    from msrest.polling import LROPoller
    from azure.common import AzureMissingResourceHttpError, AzureHttpError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

AZURE_OBJECT_CLASS = 'WebApp'


class AzureRMWebAppInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list'),
            return_publish_profile=dict(type='bool', default=False),
        )

        self.results = dict(
            changed=False,
            webapps=[],
        )

        self.name = None
        self.resource_group = None
        self.tags = None
        self.return_publish_profile = False

        self.framework_names = ['net_framework', 'java', 'php', 'node', 'python', 'dotnetcore', 'ruby']

        super(AzureRMWebAppInfo, self).__init__(self.module_arg_spec,
                                                supports_tags=False,
                                                facts_module=True)

    def exec_module(self, **kwargs):
        is_old_facts = self.module._name == 'azure_rm_webapp_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_webapp_facts' module has been renamed to 'azure_rm_webapp_info'", version='2.13')

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['webapps'] = self.list_by_name()
        elif self.resource_group:
            self.results['webapps'] = self.list_by_resource_group()
        else:
            self.results['webapps'] = self.list_all()

        return self.results

    def list_by_name(self):
        self.log('Get web app {0}'.format(self.name))
        item = None
        result = []

        try:
            item = self.web_client.web_apps.get(self.resource_group, self.name)
        except CloudError:
            pass

        if item and self.has_tags(item.tags, self.tags):
            curated_result = self.get_curated_webapp(self.resource_group, self.name, item)
            result = [curated_result]

        return result

    def list_by_resource_group(self):
        self.log('List web apps in resource groups {0}'.format(self.resource_group))
        try:
            response = list(self.web_client.web_apps.list_by_resource_group(self.resource_group))
        except CloudError as exc:
            request_id = exc.request_id if exc.request_id else ''
            self.fail("Error listing web apps in resource groups {0}, request id: {1} - {2}".format(self.resource_group, request_id, str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                curated_output = self.get_curated_webapp(self.resource_group, item.name, item)
                results.append(curated_output)
        return results

    def list_all(self):
        self.log('List web apps in current subscription')
        try:
            response = list(self.web_client.web_apps.list())
        except CloudError as exc:
            request_id = exc.request_id if exc.request_id else ''
            self.fail("Error listing web apps, request id {0} - {1}".format(request_id, str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                curated_output = self.get_curated_webapp(item.resource_group, item.name, item)
                results.append(curated_output)
        return results

    def list_webapp_configuration(self, resource_group, name):
        self.log('Get web app {0} configuration'.format(name))

        response = []

        try:
            response = self.web_client.web_apps.get_configuration(resource_group_name=resource_group, name=name)
        except CloudError as ex:
            request_id = ex.request_id if ex.request_id else ''
            self.fail('Error getting web app {0} configuration, request id {1} - {2}'.format(name, request_id, str(ex)))

        return response.as_dict()

    def list_webapp_appsettings(self, resource_group, name):
        self.log('Get web app {0} app settings'.format(name))

        response = []

        try:
            response = self.web_client.web_apps.list_application_settings(resource_group_name=resource_group, name=name)
        except CloudError as ex:
            request_id = ex.request_id if ex.request_id else ''
            self.fail('Error getting web app {0} app settings, request id {1} - {2}'.format(name, request_id, str(ex)))

        return response.as_dict()

    def get_publish_credentials(self, resource_group, name):
        self.log('Get web app {0} publish credentials'.format(name))
        try:
            poller = self.web_client.web_apps.list_publishing_credentials(resource_group, name)
            if isinstance(poller, LROPoller):
                response = self.get_poller_result(poller)
        except CloudError as ex:
            request_id = ex.request_id if ex.request_id else ''
            self.fail('Error getting web app {0} publishing credentials - {1}'.format(request_id, str(ex)))
        return response

    def get_webapp_ftp_publish_url(self, resource_group, name):
        import xmltodict

        self.log('Get web app {0} app publish profile'.format(name))

        url = None
        try:
            content = self.web_client.web_apps.list_publishing_profile_xml_with_secrets(resource_group_name=resource_group, name=name)
            if not content:
                return url

            full_xml = ''
            for f in content:
                full_xml += f.decode()
            profiles = xmltodict.parse(full_xml, xml_attribs=True)['publishData']['publishProfile']

            if not profiles:
                return url

            for profile in profiles:
                if profile['@publishMethod'] == 'FTP':
                    url = profile['@publishUrl']

        except CloudError as ex:
            self.fail('Error getting web app {0} app settings'.format(name))

        return url

    def get_curated_webapp(self, resource_group, name, webapp):
        pip = self.serialize_obj(webapp, AZURE_OBJECT_CLASS)

        try:
            site_config = self.list_webapp_configuration(resource_group, name)
            app_settings = self.list_webapp_appsettings(resource_group, name)
            publish_cred = self.get_publish_credentials(resource_group, name)
            ftp_publish_url = self.get_webapp_ftp_publish_url(resource_group, name)
        except CloudError as ex:
            pass
        return self.construct_curated_webapp(webapp=pip,
                                             configuration=site_config,
                                             app_settings=app_settings,
                                             deployment_slot=None,
                                             ftp_publish_url=ftp_publish_url,
                                             publish_credentials=publish_cred)

    def construct_curated_webapp(self,
                                 webapp,
                                 configuration=None,
                                 app_settings=None,
                                 deployment_slot=None,
                                 ftp_publish_url=None,
                                 publish_credentials=None):
        curated_output = dict()
        curated_output['id'] = webapp['id']
        curated_output['name'] = webapp['name']
        curated_output['resource_group'] = webapp['properties']['resourceGroup']
        curated_output['location'] = webapp['location']
        curated_output['plan'] = webapp['properties']['serverFarmId']
        curated_output['tags'] = webapp.get('tags', None)

        # important properties from output. not match input arguments.
        curated_output['app_state'] = webapp['properties']['state']
        curated_output['availability_state'] = webapp['properties']['availabilityState']
        curated_output['default_host_name'] = webapp['properties']['defaultHostName']
        curated_output['host_names'] = webapp['properties']['hostNames']
        curated_output['enabled'] = webapp['properties']['enabled']
        curated_output['enabled_host_names'] = webapp['properties']['enabledHostNames']
        curated_output['host_name_ssl_states'] = webapp['properties']['hostNameSslStates']
        curated_output['outbound_ip_addresses'] = webapp['properties']['outboundIpAddresses']

        # curated site_config
        if configuration:
            curated_output['frameworks'] = []
            for fx_name in self.framework_names:
                fx_version = configuration.get(fx_name + '_version', None)
                if fx_version:
                    fx = {
                        'name': fx_name,
                        'version': fx_version
                    }
                    # java container setting
                    if fx_name == 'java':
                        if configuration['java_container'] and configuration['java_container_version']:
                            settings = {
                                'java_container': configuration['java_container'].lower(),
                                'java_container_version': configuration['java_container_version']
                            }
                            fx['settings'] = settings

                    curated_output['frameworks'].append(fx)

            # linux_fx_version
            if configuration.get('linux_fx_version', None):
                tmp = configuration.get('linux_fx_version').split("|")
                if len(tmp) == 2:
                    curated_output['frameworks'].append({'name': tmp[0].lower(), 'version': tmp[1]})

        # curated app_settings
        if app_settings and app_settings.get('properties', None):
            curated_output['app_settings'] = dict()
            for item in app_settings['properties']:
                curated_output['app_settings'][item] = app_settings['properties'][item]

        # curated deploymenet_slot
        if deployment_slot:
            curated_output['deployment_slot'] = deployment_slot

        # ftp_publish_url
        if ftp_publish_url:
            curated_output['ftp_publish_url'] = ftp_publish_url

        # curated publish credentials
        if publish_credentials and self.return_publish_profile:
            curated_output['publishing_username'] = publish_credentials.publishing_user_name
            curated_output['publishing_password'] = publish_credentials.publishing_password
        return curated_output


def main():
    AzureRMWebAppInfo()


if __name__ == '__main__':
    main()



from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest

from awx.main.models import Label


@pytest.mark.django_db
def test_create_label(run_module, admin_user, organization):
    result = run_module('tower_label', dict(
        name='test-label',
        organization=organization.name
    ), admin_user)
    assert not result.get('failed'), result.get('msg', result)
    assert result.get('changed', False)

    assert Label.objects.get(name='test-label').organization == organization


@pytest.mark.django_db
def test_create_label_using_org_id(run_module, admin_user, organization):
    result = run_module('tower_label', dict(
        name='test-label',
        organization=organization.id
    ), admin_user)
    assert not result.get('failed'), result.get('msg', result)
    assert result.get('changed', False)

    assert Label.objects.get(name='test-label').organization == organization


@pytest.mark.django_db
def test_modify_label(run_module, admin_user, organization):
    label = Label.objects.create(name='test-label', organization=organization)

    result = run_module('tower_label', dict(
        name='test-label',
        new_name='renamed-label',
        organization=organization.name
    ), admin_user)
    assert not result.get('failed'), result.get('msg', result)
    assert result.get('changed', False)

    label.refresh_from_db()
    assert label.organization == organization
    assert label.name == 'renamed-label'



# -*- coding: utf-8 -*-
# (c) 2015, Allen Sanabria <asanabria@linuxdynasty.org>
#
# This file is part of Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

try:
    import boto3
    import botocore
    HAS_BOTO3 = True
except Exception:
    HAS_BOTO3 = False

import pytest

from ansible_collections.amazon.aws.tests.unit.compat import unittest
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AWSRetry

if not HAS_BOTO3:
    pytestmark = pytest.mark.skip("test_aws.py requires the python modules 'boto3' and 'botocore'")


class RetryTestCase(unittest.TestCase):

    def test_no_failures(self):
        self.counter = 0

        @AWSRetry.backoff(tries=2, delay=0.1)
        def no_failures():
            self.counter += 1

        r = no_failures()
        self.assertEqual(self.counter, 1)

    def test_extend_boto3_failures(self):
        self.counter = 0
        err_msg = {'Error': {'Code': 'MalformedPolicyDocument'}}

        @AWSRetry.backoff(tries=2, delay=0.1, catch_extra_error_codes=['MalformedPolicyDocument'])
        def extend_failures():
            self.counter += 1
            if self.counter < 2:
                raise botocore.exceptions.ClientError(err_msg, 'You did something wrong.')
            else:
                return 'success'

        r = extend_failures()
        self.assertEqual(r, 'success')
        self.assertEqual(self.counter, 2)

    def test_retry_once(self):
        self.counter = 0
        err_msg = {'Error': {'Code': 'InternalFailure'}}

        @AWSRetry.backoff(tries=2, delay=0.1)
        def retry_once():
            self.counter += 1
            if self.counter < 2:
                raise botocore.exceptions.ClientError(err_msg, 'Something went wrong!')
            else:
                return 'success'

        r = retry_once()
        self.assertEqual(r, 'success')
        self.assertEqual(self.counter, 2)

    def test_reached_limit(self):
        self.counter = 0
        err_msg = {'Error': {'Code': 'RequestLimitExceeded'}}

        @AWSRetry.backoff(tries=4, delay=0.1)
        def fail():
            self.counter += 1
            raise botocore.exceptions.ClientError(err_msg, 'toooo fast!!')

        # with self.assertRaises(botocore.exceptions.ClientError):
        try:
            fail()
        except Exception as e:
            self.assertEqual(e.response['Error']['Code'], 'RequestLimitExceeded')
        self.assertEqual(self.counter, 4)

    def test_unexpected_exception_does_not_retry(self):
        self.counter = 0
        err_msg = {'Error': {'Code': 'AuthFailure'}}

        @AWSRetry.backoff(tries=4, delay=0.1)
        def raise_unexpected_error():
            self.counter += 1
            raise botocore.exceptions.ClientError(err_msg, 'unexpected error')

        # with self.assertRaises(botocore.exceptions.ClientError):
        try:
            raise_unexpected_error()
        except Exception as e:
            self.assertEqual(e.response['Error']['Code'], 'AuthFailure')

        self.assertEqual(self.counter, 1)
